#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from os import path
from os import remove
import re
import tempfile

from oslo_serialization import jsonutils

from sahara_tempest_plugin.tests.cli import base


class SaharaPluginCLITest(base.ClientTestBase):

    def openstack_plugin_list(self):
        self.assertTableStruct(self.listing_result('plugin list'), [
            'Name',
            'Versions'
        ])

    def openstack_plugin_show(self):
        list_plugin = self.listing_result('plugin list')
        name = [p['Name'] for p in list_plugin]
        if len(name) == 0:
            raise self.skipException('No plugins to show')
        self.assertTableStruct(
            self.listing_result(''.join(['plugin show ', name[0]])), [
                'Field',
                'Value'
            ])

    def openstack_plugin_configs_get(self):
        list_plugin = self.listing_result('plugin list')
        name = [p['Name'] for p in list_plugin]
        version = [p['Versions'] for p in list_plugin]
        if len(name) == 0:
            raise self.skipException('No plugin to get configs')
        plugin_name = name[0]
        plugin_version = version[0].split(',')[0].strip()

        configs_file = '%s-%s' % (plugin_name, plugin_version)
        if path.exists(configs_file):
            remove(configs_file)

        outmsg = self.openstack('dataprocessing plugin configs get',
                                params='%s %s' % (plugin_name, plugin_version))
        outfile_match = re.search('configs was saved in "(.+)"', outmsg)
        if outfile_match:
            configs_file = outfile_match.group(1)
        else:
            configs_file = '%s-%s' % (plugin_name, plugin_version)

        result = path.exists(configs_file)
        self.assertTrue(result)
        remove(configs_file)

    def openstack_plugin_update(self):
        # check plugin list and 'fake' is available
        list_plugin = self.listing_result('plugin list')
        name = [p['Name'] for p in list_plugin]
        if len(name) == 0:
            raise self.skipException('No plugin to update')
        if 'fake' not in name:
            raise self.skipException('fake plugin is unavailable')

        # update value of "hidden:status" to False
        update_dict = {'plugin_labels': {'hidden': {'status': False}}}
        self._update_with_json_file(update_dict)

        # update value and verified it
        update_dict = {'plugin_labels': {'hidden': {'status': True}}}
        update_info = self._update_with_json_file(update_dict)
        update_dict = jsonutils.loads(update_info)
        self.assertTrue(update_dict['Plugin: hidden'])

    def _update_with_json_file(self, update_dict):
        update_info = jsonutils.dumps(update_dict)
        tmp_file = tempfile.mkstemp(suffix='.json')[1]
        with open(tmp_file, 'w+') as fd:
            fd.write(update_info)
        update_result = self.openstack('dataprocessing plugin '
                                       'update -f json fake',
                                       params=tmp_file)
        remove(tmp_file)
        return update_result



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.clients import base


class DataSourceTest(base.BaseDataProcessingTest):
    def _check_data_source_create(self, source_body):
        source_name = data_utils.rand_name('sahara-data-source')
        # create data source
        resp_body = self.create_data_source(source_name, **source_body)
        # check that source created successfully
        self.assertEqual(source_name, resp_body.name)
        if source_body['type'] == 'swift':
            source_body = self.swift_data_source
        self.assertDictContainsSubset(source_body, resp_body.__dict__)

        return resp_body.id, source_name

    def _check_data_source_list(self, source_id, source_name):
        # check for data source in list
        source_list = self.client.data_sources.list()
        sources_info = [(source.id, source.name) for source in source_list]
        self.assertIn((source_id, source_name), sources_info)

    def _check_data_source_get(self, source_id, source_name, source_body):
        # check data source fetch by id
        source = self.client.data_sources.get(source_id)
        self.assertEqual(source_name, source.name)
        self.assertDictContainsSubset(source_body, source.__dict__)

    def _check_data_source_update(self, source_id):
        values = {
            'name': data_utils.rand_name('updated-sahara-data-source'),
            'description': 'description',
            'type': 'hdfs',
            'url': 'hdfs://user/foo'
        }

        source = self.client.data_sources.update(source_id, values)

        self.assertDictContainsSubset(values, source.data_source)

    def _check_data_source_delete(self, source_id):
        # delete data source
        self.client.data_sources.delete(source_id)
        # check that data source really deleted
        source_list = self.client.data_sources.list()
        self.assertNotIn(source_id, [source.id for source in source_list])

    def test_swift_data_source(self):
        # Create extra self.swift_data_source variable to use for comparison to
        # data source response body because response body has no 'credentials'
        # field.
        self.swift_data_source = self.swift_data_source_with_creds.copy()
        del self.swift_data_source['credentials']
        source_id, source_name = self._check_data_source_create(
            self.swift_data_source_with_creds)
        self._check_data_source_list(source_id, source_name)
        self._check_data_source_get(source_id, source_name,
                                    self.swift_data_source)
        self._check_data_source_delete(source_id)

    def test_local_hdfs_data_source(self):
        source_id, source_name = self._check_data_source_create(
            self.local_hdfs_data_source)
        self._check_data_source_list(source_id, source_name)
        self._check_data_source_get(source_id, source_name,
                                    self.local_hdfs_data_source)
        self._check_data_source_delete(source_id)

    def test_external_hdfs_data_source(self):
        source_id, source_name = self._check_data_source_create(
            self.external_hdfs_data_source)
        self._check_data_source_list(source_id, source_name)
        self._check_data_source_get(source_id, source_name,
                                    self.external_hdfs_data_source)
        self._check_data_source_update(source_id)
        self._check_data_source_delete(source_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sahara_tempest_plugin.tests.cli import base
from tempest.lib.common.utils import data_utils


class SaharaDataSourceCLITest(base.ClientTestBase):

    def openstack_data_source_list(self):
        self.assertTableStruct(self.listing_result('data source list'), [
            'Name',
            'Type'
        ])

    def openstack_data_source_create(self):
        data_source_name = data_utils.rand_name('data-source')
        flag = ("%(name)s %(type)s %(url)s %(user)s %(pass)s"
                % {'type': '--type hdfs',
                   'url': '--url hdfs://demo/input.tar.gz',
                   'user': ' --user cli-test',
                   'pass': '--password cli',
                   'name': data_source_name})
        self.assertTableStruct(
            self.listing_result('data source create %s' % flag),
            [
                'Field',
                'Value'
            ])
        return data_source_name

    def openstack_data_source_show(self, data_source_name):
        self.find_in_listing(
            self.listing_result('data source show %s' % data_source_name),
            data_source_name)

    def openstack_data_source_update(self, data_source_name):
        self.assertTableStruct(
            self.listing_result('data source update %s' % data_source_name),
            [
                'Field',
                'Value'
            ])

    def openstack_data_source_delete(self, data_source_name):
        self.check_if_delete('data source', data_source_name)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.clients import base


CONF = config.CONF


class JobTest(base.BaseDataProcessingTest):

    def _get_job_template_client(self):
        if CONF.data_processing.api_version_saharaclient == '1.1':
            job_template_client_class = self.client.jobs
        else:
            job_template_client_class = self.client.job_templates
        return job_template_client_class

    def _get_job_template_items(self, job_template_object):
        if CONF.data_processing.api_version_saharaclient == '1.1':
            return job_template_object.job
        else:
            return job_template_object.job_template

    def _check_create_job(self):
        job_binary = {
            'name': data_utils.rand_name('sahara-job-binary'),
            'url': 'swift://sahara-container.sahara/example.jar',
            'description': 'Test job binary',
            'extra': {
                'user': 'test',
                'password': '123'
            }
        }
        # create job_binary
        job_binary = self.create_job_binary(**job_binary)

        self.job = {
            'job_type': 'Pig',
            'mains': [job_binary.id]
        }
        job_name = data_utils.rand_name('sahara-job')
        # create job
        job = self.create_job(job_name, **self.job)
        # check that job created successfully
        self.assertEqual(job_name, job.name)

        return job.id, job.name

    def _check_job_list(self, job_id, job_name):
        # check for job in list
        job_list = self._get_job_template_client().list()
        jobs_info = [(job.id, job.name) for job in job_list]
        self.assertIn((job_id, job_name), jobs_info)

    def _check_get_job(self, job_id, job_name):
        # check job fetch by id
        job = self._get_job_template_client().get(job_id)
        self.assertEqual(job_name, job.name)

    def _check_job_update(self, job_id):
        # check updating of job
        values = {
            'name': data_utils.rand_name('updated-sahara-job'),
            'description': 'description'

        }
        job = self._get_job_template_client().update(job_id, **values)
        self.assertDictContainsSubset(values,
                                      self._get_job_template_items(job))

    def _check_delete_job(self, job_id):
        # delete job by id
        self._get_job_template_client().delete(job_id)
        # check that job really deleted
        job_list = self._get_job_template_client().list()
        self.assertNotIn(job_id, [job.id for job in job_list])

    def test_job(self):
        job_id, job_name = self._check_create_job()
        self._check_job_list(job_id, job_name)
        self._check_get_job(job_id, job_name)
        self._check_job_update(job_id)
        self._check_delete_job(job_id)



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import OrderedDict
import copy

from tempest import config


CONF = config.CONF


"""Default templates.
There should always be at least a master1 and a worker1 node
group template."""
BASE_VANILLA_DESC = {
    'NODES': {
        'master1': {
            'count': 1,
            'node_processes': ['namenode', 'resourcemanager',
                               'hiveserver']
        },
        'master2': {
            'count': 1,
            'node_processes': ['oozie', 'historyserver',
                               'secondarynamenode']
        },
        'worker1': {
            'count': 1,
            'node_processes': ['datanode', 'nodemanager'],
            'node_configs': {
                'MapReduce': {
                    'yarn.app.mapreduce.am.resource.mb': 256,
                    'yarn.app.mapreduce.am.command-opts': '-Xmx256m'
                },
                'YARN': {
                    'yarn.scheduler.minimum-allocation-mb': 256,
                    'yarn.scheduler.maximum-allocation-mb': 1024,
                    'yarn.nodemanager.vmem-check-enabled': False
                }
            }
        }
    },
    'cluster_configs': {
        'HDFS': {
            'dfs.replication': 1
        }
    }
}

BASE_SPARK_DESC = {
    'NODES': {
        'master1': {
            'count': 1,
            'node_processes': ['namenode', 'master']
        },
        'worker1': {
            'count': 1,
            'node_processes': ['datanode', 'slave']
        }
    },
    'cluster_configs': {
        'HDFS': {
            'dfs.replication': 1
        }
    }
}

BASE_CDH_DESC = {
    'NODES': {
        'master1': {
            'count': 1,
            'node_processes': ['CLOUDERA_MANAGER']
        },
        'master2': {
            'count': 1,
            'node_processes': ['HDFS_NAMENODE',
                               'YARN_RESOURCEMANAGER']
        },
        'master3': {
            'count': 1,
            'node_processes': ['OOZIE_SERVER', 'YARN_JOBHISTORY',
                               'HDFS_SECONDARYNAMENODE',
                               'HIVE_METASTORE', 'HIVE_SERVER2']
        },
        'worker1': {
            'count': 1,
            'node_processes': ['YARN_NODEMANAGER', 'HDFS_DATANODE']
        }
    },
    'cluster_configs': {
        'HDFS': {
            'dfs_replication': 1
        }
    }
}

BASE_AMBARI_HDP_DESC = {
    'NODES': {
        'master1': {
            'count': 1,
            'node_processes': ['Ambari', 'MapReduce History Server',
                               'Spark History Server', 'NameNode',
                               'ResourceManager', 'SecondaryNameNode',
                               'YARN Timeline Server', 'ZooKeeper',
                               'Kafka Broker']
        },
        'master2': {
            'count': 1,
            'node_processes': ['Hive Metastore', 'HiveServer', 'Oozie']
        },
        'worker1': {
            'count': 3,
            'node_processes': ['DataNode', 'NodeManager']
        }
    },
    'cluster_configs': {
        'HDFS': {
            'dfs.datanode.du.reserved': 0
        }
    }
}

BASE_MAPR_DESC = {
    'NODES': {
        'master1': {
            'count': 1,
            'node_processes': ['Metrics', 'Webserver', 'ZooKeeper',
                               'HTTPFS', 'Oozie', 'FileServer', 'CLDB',
                               'Flume', 'Hue', 'NodeManager', 'HistoryServer',
                               'ResourceManager', 'HiveServer2',
                               'HiveMetastore',
                               'Sqoop2-Client', 'Sqoop2-Server']
        },
        'worker1': {
            'count': 1,
            'node_processes': ['NodeManager', 'FileServer']
        }
    }
}

BASE_STORM_DESC = {
    'NODES': {
        'master1': {
            'count': 1,
            'node_processes': ['nimbus']
        },
        'master2': {
            'count': 1,
            'node_processes': ['zookeeper']
        },
        'worker1': {
            'count': 1,
            'node_processes': ['supervisor']
        }
    }
}


DEFAULT_TEMPLATES = {
    'fake': OrderedDict([
        ('0.1', {
            'NODES': {
                'master1': {
                    'count': 1,
                    'node_processes': ['namenode', 'jobtracker']
                },
                'worker1': {
                    'count': 1,
                    'node_processes': ['datanode', 'tasktracker'],
                }
            }
        })
    ]),
    'vanilla': OrderedDict([
        ('2.7.1', copy.deepcopy(BASE_VANILLA_DESC)),
        ('2.7.5', copy.deepcopy(BASE_VANILLA_DESC)),
        ('2.8.2', copy.deepcopy(BASE_VANILLA_DESC))
    ]),
    'ambari': OrderedDict([
        ('2.3', copy.deepcopy(BASE_AMBARI_HDP_DESC)),
        ('2.4', copy.deepcopy(BASE_AMBARI_HDP_DESC)),
        ('2.5', copy.deepcopy(BASE_AMBARI_HDP_DESC)),
        ('2.6', copy.deepcopy(BASE_AMBARI_HDP_DESC))
    ]),
    'spark': OrderedDict([
        ('1.3.1', copy.deepcopy(BASE_SPARK_DESC)),
        ('1.6.0', copy.deepcopy(BASE_SPARK_DESC)),
        ('2.1.0', copy.deepcopy(BASE_SPARK_DESC)),
        ('2.2', copy.deepcopy(BASE_SPARK_DESC)),
        ('2.3', copy.deepcopy(BASE_SPARK_DESC))
    ]),
    'cdh': OrderedDict([
        ('5.4.0', copy.deepcopy(BASE_CDH_DESC)),
        ('5.5.0', copy.deepcopy(BASE_CDH_DESC)),
        ('5.7.0', copy.deepcopy(BASE_CDH_DESC)),
        ('5.9.0', copy.deepcopy(BASE_CDH_DESC)),
        ('5.11.0', copy.deepcopy(BASE_CDH_DESC)),
        ('5.13.0', copy.deepcopy(BASE_CDH_DESC))
    ]),
    'mapr': OrderedDict([
        ('5.1.0.mrv2', copy.deepcopy(BASE_MAPR_DESC)),
        ('5.2.0.mrv2', copy.deepcopy(BASE_MAPR_DESC))
    ]),
    'storm': OrderedDict([
        ('1.0.1', copy.deepcopy(BASE_STORM_DESC)),
        ('1.1.0', copy.deepcopy(BASE_STORM_DESC)),
        ('1.2', copy.deepcopy(BASE_STORM_DESC))
    ])
}


def get_plugin_data(plugin_name, plugin_version):
    return DEFAULT_TEMPLATES[plugin_name][plugin_version]


def get_default_plugin():
    """Returns the default plugin used for testing."""
    enabled_plugins = CONF.data_processing_feature_enabled.plugins

    if len(enabled_plugins) == 0:
        return None

    # NOTE(raissa) if fake is available, use it first.
    # this is to reduce load and should be removed
    # once the fake plugin is no longer needed
    if 'fake' in enabled_plugins:
        return 'fake'

    for plugin in enabled_plugins:
        if plugin in DEFAULT_TEMPLATES.keys():
            break
    else:
        plugin = ''
    return plugin


def get_default_version(plugin):
    """Returns the default plugin version used for testing.

    This is gathered separately from the plugin to allow
    the usage of plugin name in skip_checks. This method is
    rather invoked into resource_setup, which allows API calls
    and exceptions.
    """
    default_plugin_name = get_default_plugin()

    if not (plugin and default_plugin_name):
        return None

    for version in DEFAULT_TEMPLATES[default_plugin_name].keys():
        if version in plugin['versions']:
            break
    else:
        version = None

    return version


def get_node_group_template(nodegroup='worker1',
                            default_version=None,
                            floating_ip_pool=None,
                            api_version='1.1'):
    """Returns a node group template for the default plugin."""
    try:
        flavor = CONF.compute.flavor_ref
        default_plugin_name = get_default_plugin()
        plugin_data = (
            get_plugin_data(default_plugin_name, default_version)
        )
        nodegroup_data = plugin_data['NODES'][nodegroup]
        node_group_template = {
            'description': 'Test node group template',
            'plugin_name': default_plugin_name,
            'node_processes': nodegroup_data['node_processes'],
            'flavor_id': flavor,
            'floating_ip_pool': floating_ip_pool,
            'node_configs': nodegroup_data.get('node_configs', {})
        }
        if api_version == '1.1':
            node_group_template['hadoop_version'] = default_version
        else:
            node_group_template['plugin_version'] = default_version
        return node_group_template
    except (IndexError, KeyError):
        return None


def get_cluster_template(node_group_template_ids=None,
                         default_version=None,
                         api_version='1.1'):
    """Returns a cluster template for the default plugin.

    node_group_template_ids contains the type and ID of pre-defined
    node group templates that have to be used in the cluster template
    (instead of dynamically defining them with 'node_processes').
    """
    flavor = CONF.compute.flavor_ref
    default_plugin_name = get_default_plugin()

    if node_group_template_ids is None:
        node_group_template_ids = {}
    try:
        plugin_data = (
            get_plugin_data(default_plugin_name, default_version)
        )

        all_node_groups = []
        for ng_name, ng_data in plugin_data['NODES'].items():
            node_group = {
                'name': '%s-node' % (ng_name),
                'flavor_id': flavor,
                'count': ng_data['count']
            }
            if ng_name in node_group_template_ids.keys():
                # node group already defined, use it
                node_group['node_group_template_id'] = (
                    node_group_template_ids[ng_name]
                )
            else:
                # node_processes list defined on-the-fly
                node_group['node_processes'] = ng_data['node_processes']
            if 'node_configs' in ng_data:
                node_group['node_configs'] = ng_data['node_configs']
            all_node_groups.append(node_group)

        cluster_template = {
            'description': 'Test cluster template',
            'plugin_name': default_plugin_name,
            'cluster_configs': plugin_data.get('cluster_configs', {}),
            'node_groups': all_node_groups,
        }
        if api_version == '1.1':
            cluster_template['hadoop_version'] = default_version
        else:
            cluster_template['plugin_version'] = default_version
        return cluster_template
    except (IndexError, KeyError):
        return None



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from sahara_tests.scenario import base


@base.track_result("Check sentry")
def check(self):
    nodes = self._get_nodes_with_process('HBASE_MASTER')
    for node in nodes:
        node_ip = node['management_ip']
        conffile_dir = self._run_command_on_node(
            node_ip,
            'sudo find / -name "*-sentry-SENTRY_SERVER" | head -1'
        )
        self._run_command_on_node(
            node_ip, 'sudo cp %s/sentry-site.xml /tmp/sentry-site.xml'
                     % conffile_dir)
        self._run_command_on_node(node_ip,
                                  'sudo chmod 664 /tmp/sentry-site.xml')
        psql_jar = self._run_command_on_node(
            node_ip, 'ls /usr/share/cmf/lib/postgresql* | head -1')
        self._run_command_on_node(node_ip,
                                  'export HADOOP_CLASSPATH=:%s' % psql_jar)
        self._run_command_on_node(
            node_ip,
            'sentry --command schema-tool -conffile /tmp/sentry-site.xml '
            '-dbType postgres -info')



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import testcase as tc

from tempest import config
from tempest.lib import decorators
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.api import base as dp_base


CONF = config.CONF


class JobBinaryInternalTest(dp_base.BaseDataProcessingTest):
    # Link to the API documentation is https://developer.openstack.org/
    # api-ref/data-processing/#job-binary-internals

    @classmethod
    def skip_checks(cls):
        super(JobBinaryInternalTest, cls).skip_checks()
        if CONF.data_processing.use_api_v2:
            raise cls.skipException('Job binaries stored on the internal db '
                                    'are available only with API v1.1')

    @classmethod
    def resource_setup(cls):
        super(JobBinaryInternalTest, cls).resource_setup()
        cls.job_binary_internal_data = 'Some script may be data'

    def _create_job_binary_internal(self, binary_name=None):
        """Creates Job Binary Internal with optional name specified.

        It puts data into Sahara database and ensures job binary internal name.
        Returns id and name of created job binary internal.
        """
        if not binary_name:
            # generate random name if it's not specified
            binary_name = data_utils.rand_name('sahara-job-binary-internal')

        # create job binary internal
        resp_body = (
            self.create_job_binary_internal(binary_name,
                                            self.job_binary_internal_data))

        # ensure that job binary internal created successfully
        self.assertEqual(binary_name, resp_body['name'])

        return resp_body['id'], binary_name

    @tc.attr('smoke')
    @decorators.idempotent_id('249c4dc2-946f-4939-83e6-212ddb6ea0be')
    def test_job_binary_internal_create(self):
        self._create_job_binary_internal()

    @tc.attr('smoke')
    @decorators.idempotent_id('1e3c2ecd-5673-499d-babe-4fe2fcdf64ee')
    def test_job_binary_internal_list(self):
        binary_info = self._create_job_binary_internal()

        # check for job binary internal in list
        binaries = self.client.list_job_binary_internals()['binaries']
        binaries_info = [(binary['id'], binary['name']) for binary in binaries]
        self.assertIn(binary_info, binaries_info)

    @tc.attr('smoke')
    @decorators.idempotent_id('a2046a53-386c-43ab-be35-df54b19db776')
    def test_job_binary_internal_get(self):
        binary_id, binary_name = self._create_job_binary_internal()

        # check job binary internal fetch by id
        binary = self.client.get_job_binary_internal(binary_id)
        self.assertEqual(binary_name, binary['job_binary_internal']['name'])

    @tc.attr('smoke')
    @decorators.idempotent_id('b3568c33-4eed-40d5-aae4-6ff3b2ac58f5')
    def test_job_binary_internal_delete(self):
        binary_id, _ = self._create_job_binary_internal()

        # delete the job binary internal by id
        self.client.delete_job_binary_internal(binary_id)
        get_resource = self.client.get_job_binary_internal
        self.wait_for_resource_deletion(binary_id, get_resource)

        # check for job binary internal in list
        binaries = self.client.list_job_binary_internals()['binaries']
        binaries_ids = [binary['id'] for binary in binaries]
        self.assertNotIn(binary_id, binaries_ids)

    @tc.attr('smoke')
    @decorators.idempotent_id('8871f2b0-5782-4d66-9bb9-6f95bcb839ea')
    def test_job_binary_internal_get_data(self):
        binary_id, _ = self._create_job_binary_internal()

        # get data of job binary internal by id
        _, data = self.client.get_job_binary_internal_data(binary_id)
        self.assertEqual(data.decode("utf-8"), self.job_binary_internal_data)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest import config
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.clients import base


CONF = config.CONF


class JobBinariesTest(base.BaseDataProcessingTest):
    def _check_job_binary_create(self, binary_body):
        binary_name = data_utils.rand_name('sahara-job-binary')

        # create job binary
        resp_body = self.create_job_binary(binary_name, **binary_body)

        # ensure that binary created successfully
        self.assertEqual(binary_name, resp_body.name)
        if 'swift' in binary_body['url']:
            binary_body = self.swift_job_binary
        else:
            binary_body = self.internal_db_binary
        self.assertDictContainsSubset(binary_body, resp_body.__dict__)

        return resp_body.id, binary_name

    def _check_job_binary_list(self, binary_id, binary_name):
        # check for job binary in list
        binary_list = self.client.job_binaries.list()
        binaries_info = [(binary.id, binary.name) for binary in binary_list]
        self.assertIn((binary_id, binary_name), binaries_info)

    def _check_job_binary_delete(self, binary_id):
        # delete job binary by id
        self.client.job_binaries.delete(binary_id)
        # check that job binary really deleted
        binary_list = self.client.job_binaries.list()
        self.assertNotIn(binary_id, [binary.id for binary in binary_list])

    def _check_swift_job_binary_create(self):
        self.swift_job_binary_with_extra = {
            'url': 'swift://sahara-container/example.jar',
            'description': 'Test job binary',
            'extra': {
                'user': 'test',
                'password': '123'
            }
        }
        # Create extra self.swift_job_binary variable to use for comparison to
        # job binary response body because response body has no 'extra' field.
        self.swift_job_binary = self.swift_job_binary_with_extra.copy()
        del self.swift_job_binary['extra']
        return self._check_job_binary_create(self.swift_job_binary_with_extra)

    def _check_swift_job_binary_get(self, binary_id, binary_name):
        # check job binary fetch by id
        binary = self.client.job_binaries.get(binary_id)
        self.assertEqual(binary_name, binary.name)
        self.assertDictContainsSubset(self.swift_job_binary, binary.__dict__)

    def _check_swift_job_binary_update(self, binary_id):
        values = {
            'url': 'swift://user/foo',
            'description': 'description'
        }
        # check updating of job binary in swift
        binary = self.client.job_binaries.update(binary_id, values)
        self.assertDictContainsSubset(values, binary.__dict__)

    def _check_internal_db_job_binary_create(self):
        name = data_utils.rand_name('sahara-internal-job-binary')
        self.job_binary_data = b'Some data'
        job_binary_internal = (
            self.create_job_binary_internal(name, self.job_binary_data))
        self.internal_db_binary_with_extra = {
            'url': 'internal-db://%s' % job_binary_internal.id,
            'description': 'Test job binary',
            'extra': {
                'user': 'test',
                'password': '123'
            }
        }
        # Create extra self.internal_db_binary variable to use for comparison
        # to job binary response body because response body has no 'extra'
        # field.
        self.internal_db_binary = self.internal_db_binary_with_extra.copy()
        del self.internal_db_binary['extra']
        return self._check_job_binary_create(
            self.internal_db_binary_with_extra)

    def _check_internal_db_job_binary_get(self, binary_id, binary_name):
        # check job binary fetch by id
        binary = self.client.job_binaries.get(binary_id)
        self.assertEqual(binary_name, binary.name)
        self.assertDictContainsSubset(self.internal_db_binary, binary.__dict__)

    def _check_internal_db_job_binary_update(self, binary_id):
        values = {
            'description': 'description'
        }
        # check updating of job binary in internal db
        binary = self.client.job_binaries.update(binary_id, values)
        self.assertDictContainsSubset(values, binary.__dict__)

    def _check_job_binary_get_file(self, binary_id):
        data = self.client.job_binaries.get_file(binary_id)
        self.assertEqual(self.job_binary_data, data)

    def test_swift_job_binaries(self):
        binary_id, binary_name = self._check_swift_job_binary_create()
        self._check_job_binary_list(binary_id, binary_name)
        self._check_swift_job_binary_get(binary_id, binary_name)
        self._check_swift_job_binary_update(binary_id)
        self._check_job_binary_delete(binary_id)

    @testtools.skipIf(CONF.data_processing.api_version_saharaclient != '1.1',
                      'Job binaries stored on the internal db are available '
                      'only with API v1.1')
    def test_internal_job_binaries(self):
        binary_id, binary_name = self._check_internal_db_job_binary_create()
        self._check_job_binary_list(binary_id, binary_name)
        self._check_internal_db_job_binary_get(binary_id, binary_name)
        self._check_job_binary_get_file(binary_id)
        self._check_internal_db_job_binary_update(binary_id)
        self._check_job_binary_delete(binary_id)



# Copyright 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from six.moves.urllib import parse as urlparse


def url_schema_remover(url):
    """ Return the same URL without the schema.
    Example: prefix://host/path -> host/path
    """
    parsed = urlparse.urlsplit(url)
    cleaned = urlparse.urlunsplit((('',) + parsed[1:]))
    if cleaned.startswith('//'):
        cleaned = cleaned[2:]
    return cleaned



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sahara_tempest_plugin.tests.cli import base
from tempest.lib.common.utils import data_utils
import fixtures

VERIF_RESULT = '''\
Cluster "%s" health verification has been started.
'''


class SaharaClusterCLITest(base.ClientTestBase):

    def openstack_cluster_list(self):
        self.assertTableStruct(self.listing_result('cluster list'), [
            'Name',
            'Id',
            'Plugin name',
            'Plugin version',
            'Status'
        ])

    def openstack_cluster_create(self, cluster_template_name, image_name):
        cluster_name = data_utils.rand_name('cluster')
        # FIXME: this is unstable interface, but apparently there are no
        # suitable ready-to-use replacements.
        id_net_project = self.project_network['id']
        flags = ("%(name)s %(cl-tm)s %(image)s %(network)s"
                 % {'network': '--neutron-network %s' % (id_net_project),
                    'image': '--image %s' % (image_name),
                    'cl-tm': '--cluster-template %s' %
                             (cluster_template_name),
                    'name': '--name %s' % (cluster_name)})
        self.assertTableStruct(
            self.listing_result(''.join(['cluster create ', flags])), [
                'Field',
                'Value'
            ])
        self._poll_cluster_status(cluster_name)
        return cluster_name

    def openstack_cluster_delete(self, cluster_name):
        delete_cluster = self.listing_result(''.join(['cluster delete ',
                                                      cluster_name]))
        self.assertTableStruct(delete_cluster, [
            'Field',
            'Value'
        ])

    def openstack_cluster_show(self, cluster_name):
        self.find_in_listing(
            self.listing_result(''.join(['cluster show ', cluster_name])),
            cluster_name)

    def openstack_cluster_update(self, cluster_name):
        self.assertTableStruct(
            self.listing_result(''.join(['cluster update ',
                                         '--description cli-tests ',
                                         cluster_name])), [
                'Field',
                'Value'
            ])

    def openstack_cluster_verification_show(self, cluster_name):
        self.assertTableStruct(
            self.listing_result(''.join(['cluster verification --show ',
                                         cluster_name])), [
                'Field',
                'Value'
            ])

    def openstack_cluster_verification_start(self, cluster_name):
        result = self.openstack('dataprocessing cluster verification --start',
                                params=cluster_name)
        expected_result = VERIF_RESULT % cluster_name
        self.assertEqual(result, expected_result)

    def openstack_cluster_scale(self, cluster_name, ng_worker):
        with fixtures.Timeout(300, gentle=True):
            scale_cluster = self.listing_result(
                ''.join(['cluster scale --instances ', ng_worker,
                         ':2 --wait ', cluster_name]))
            self.assertTableStruct(scale_cluster, [
                'Field',
                'Value'
            ])
        self._poll_cluster_status(cluster_name)



# Copyright 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

from sahara_tests.utils import url as utils_url


class UrlUnitTest(testtools.TestCase):

    def _check_clean_url(self, url, expected_clean_url):
        """Check if the cleaned URL matches the expected one."""
        clean_url = utils_url.url_schema_remover(url)
        self.assertEqual(clean_url, expected_clean_url)

    def test_clean_url_http(self):
        self._check_clean_url('https://s3.amazonaws.com',
                              's3.amazonaws.com')

    def test_clean_url_https_longer(self):
        self._check_clean_url('https://s3.amazonaws.com/foo',
                              's3.amazonaws.com/foo')

    def test_clean_url_file(self):
        self._check_clean_url('file:///s3.amazonaws.com/bar',
                              '/s3.amazonaws.com/bar')



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time
from testtools import testcase as tc

from oslo_utils import timeutils
from saharaclient.api import base as sab
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from sahara_tempest_plugin.tests.clients import base


TEMPEST_CONF = config.CONF


class JobExecutionTest(base.BaseDataProcessingTest):
    def _check_register_image(self, image_id):
        self.client.images.update_image(
            image_id, TEMPEST_CONF.data_processing.test_ssh_user, '')
        reg_image = self.client.images.get(image_id)

        self.assertDictContainsSubset(
            {'_sahara_username': TEMPEST_CONF.data_processing.test_ssh_user},
            reg_image.metadata)

    def _check_image_get(self, image_id):
        image = self.client.images.get(image_id)

        self.assertEqual(image_id, image.id)

    def _check_image_list(self, image_id):
        # check for image in list
        image_list = self.client.images.list()
        images_info = [image.id for image in image_list]

        self.assertIn(image_id, images_info)

    def _check_adding_tags(self, image_id):
        # adding new tags
        self.client.images.update_tags(image_id, ['fake', '0.1'])
        image = self.client.images.get(image_id)

        self.assertDictContainsSubset({'_sahara_tag_fake': 'True',
                                       '_sahara_tag_0.1': 'True'},
                                      image.metadata)

    def _check_deleting_tags(self, image_id):
        # deleting tags
        self.client.images.update_tags(image_id, [])
        image = self.client.images.get(image_id)

        self.assertNotIn('_sahara_tag_fake', image.metadata)
        self.assertNotIn('_sahara_tag_0.1', image.metadata)

    def _check_unregister_image(self, image_id):
        # unregister image
        self.client.images.unregister_image(image_id)

        # check that image really unregistered
        image_list = self.client.images.list()
        self.assertNotIn(image_id, [image.id for image in image_list])

    def _check_cluster_create(self):
        worker = self.create_node_group_template(
            data_utils.rand_name('sahara-ng-template'), **self.worker_template)

        master = self.create_node_group_template(
            data_utils.rand_name('sahara-ng-template'), **self.master_template)

        cluster_templ = self.cluster_template.copy()
        cluster_templ['node_groups'] = [
            {
                'name': 'master',
                'node_group_template_id': master.id,
                'count': 1
            },
            {
                'name': 'worker',
                'node_group_template_id': worker.id,
                'count': 3
            }
        ]
        if TEMPEST_CONF.service_available.neutron:
            cluster_templ['net_id'] = self.get_private_network_id()

        cluster_template = self.create_cluster_template(
            data_utils.rand_name('sahara-cluster-template'), **cluster_templ)
        cluster_name = data_utils.rand_name('sahara-cluster')
        self.cluster_info = {
            'name': cluster_name,
            'plugin_name': 'fake',
            'cluster_template_id': cluster_template.id,
            'default_image_id': self.test_image_id
        }
        plugin_version_option = 'plugin_version'
        if CONF.data_processing.api_version_saharaclient == '1.1':
            plugin_version_option = 'hadoop_version'
        self.cluster_info[plugin_version_option] = '0.1'

        # create cluster
        cluster = self.create_cluster(**self.cluster_info)

        # wait until cluster moves to active state
        self.check_cluster_active(cluster.id)

        # check that cluster created successfully
        self.assertEqual(cluster_name, cluster.name)
        self.assertDictContainsSubset(self.cluster_info, cluster.__dict__)

        return cluster.id, cluster.name

    def _check_cluster_list(self, cluster_id, cluster_name):
        # check for cluster in list
        cluster_list = self.client.clusters.list()
        clusters_info = [(clust.id, clust.name) for clust in cluster_list]
        self.assertIn((cluster_id, cluster_name), clusters_info)

    def _check_cluster_get(self, cluster_id, cluster_name):
        # check cluster fetch by id
        cluster = self.client.clusters.get(cluster_id)
        self.assertEqual(cluster_name, cluster.name)
        self.assertDictContainsSubset(self.cluster_info, cluster.__dict__)

    def _check_cluster_update(self, cluster_id):
        values = {
            'name': data_utils.rand_name('updated-sahara-cluster'),
            'description': 'description'
        }
        # check updating of cluster
        cluster = self.client.clusters.update(cluster_id)
        self.assertDictContainsSubset(values, cluster.__dict__)

    def _check_cluster_scale(self, cluster_id):
        big_worker = self.create_node_group_template(
            data_utils.rand_name('sahara-ng-template'), **self.worker_template)

        scale_body = {
            'resize_node_groups': [
                {
                    'count': 2,
                    'name': 'worker'
                },
                {
                    "count": 2,
                    "name": 'master'
                }
            ],
            'add_node_groups': [
                {
                    'count': 1,
                    'name': 'big-worker',
                    'node_group_template_id': big_worker.id

                }
            ]
        }

        self.client.clusters.scale(cluster_id, scale_body)
        self.check_cluster_active(cluster_id)

        cluster = self.client.clusters.get(cluster_id)
        for ng in cluster.node_groups:
            if ng['name'] == scale_body['resize_node_groups'][0]['name']:
                self.assertDictContainsSubset(
                    scale_body['resize_node_groups'][0], ng)
            elif ng['name'] == scale_body['resize_node_groups'][1]['name']:
                self.assertDictContainsSubset(
                    scale_body['resize_node_groups'][1], ng)
            elif ng['name'] == scale_body['add_node_groups'][0]['name']:
                self.assertDictContainsSubset(
                    scale_body['add_node_groups'][0], ng)

    def _check_cluster_delete(self, cluster_id):
        self.client.clusters.delete(cluster_id)

        # check that cluster moved to deleting state
        cluster = self.client.clusters.get(cluster_id)
        self.assertEqual('Deleting', cluster.status)

        timeout = TEMPEST_CONF.data_processing.cluster_timeout
        s_time = timeutils.utcnow()
        while timeutils.delta_seconds(s_time, timeutils.utcnow()) < timeout:
            try:
                self.client.clusters.get(cluster_id)
            except sab.APIException:
                # cluster is deleted
                return
            time.sleep(TEMPEST_CONF.data_processing.request_timeout)

        raise exceptions.TimeoutException('Cluster failed to terminate'
                                          'in %d seconds.' % timeout)

    def _check_job_execution_create(self, cluster_id):
        # create swift container
        container_name = data_utils.rand_name('test-container')
        self.create_container(container_name)

        # create input data source
        input_file_name = data_utils.rand_name('input')
        self.object_client.create_object(container_name, input_file_name,
                                         'some-data')

        input_file_url = 'swift://%s/%s' % (container_name, input_file_name)
        input_source_name = data_utils.rand_name('input-data-source')
        input_source = self.create_data_source(
            input_source_name, input_file_url, '', 'swift',
            {'user': 'test', 'password': '123'})

        # create output data source
        output_dir_name = data_utils.rand_name('output')
        output_dir_url = 'swift://%s/%s' % (container_name, output_dir_name)
        output_source_name = data_utils.rand_name('output-data-source')
        output_source = self.create_data_source(
            output_source_name, output_dir_url, '', 'swift',
            {'user': 'test', 'password': '123'})

        job_binary = {
            'name': data_utils.rand_name('sahara-job-binary'),
            'url': input_file_url,
            'description': 'Test job binary',
            'extra': {
                'user': 'test',
                'password': '123'
            }
        }
        # create job_binary
        job_binary = self.create_job_binary(**job_binary)

        # create job
        job_name = data_utils.rand_name('test-job')
        job = self.create_job(job_name, 'Pig', [job_binary.id])

        self.job_exec_info = {
            'job_id': job.id,
            'cluster_id': cluster_id,
            'input_id': input_source.id,
            'output_id': output_source.id,
            'configs': {}
        }
        # create job execution
        job_execution = self.create_job_execution(**self.job_exec_info)

        return job_execution.id

    def _check_job_execution_list(self, job_exec_id):
        # check for job_execution in list
        job_exec_list = self.client.job_executions.list()
        self.assertIn(job_exec_id, [job_exec.id for job_exec in job_exec_list])

    def _check_job_execution_get(self, job_exec_id):
        # check job_execution fetch by id
        job_exec = self.client.job_executions.get(job_exec_id)
        # Create extra cls.swift_job_binary variable to use for comparison to
        # job binary response body because response body has no 'extra' field.
        job_exec_info = self.job_exec_info.copy()
        del job_exec_info['configs']
        self.assertDictContainsSubset(job_exec_info, job_exec.__dict__)

    def _check_job_execution_update(self, job_exec_id):
        values = {
            'is_public': True
        }
        job_exec = self.client.job_executions.update(job_exec_id, **values)
        self.assertDictContainsSubset(values, job_exec.__dict__)

    def _check_job_execution_delete(self, job_exec_id):
        # delete job_execution by id
        self.client.job_executions.delete(job_exec_id)
        # check that job_execution really deleted
        job_exec_list = self.client.jobs.list()
        self.assertNotIn(job_exec_id, [job_exec.id for
                                       job_exec in job_exec_list])

    @decorators.skip_because(bug="1430252")
    @tc.attr('slow')
    def test_job_executions(self):
        image_id = self.test_image_id
        self._check_register_image(image_id)
        self._check_image_get(image_id)
        self._check_image_list(image_id)
        self._check_adding_tags(image_id)

        cluster_id, cluster_name = self._check_cluster_create()
        self._check_cluster_list(cluster_id, cluster_name)
        self._check_cluster_get(cluster_id, cluster_name)
        self._check_cluster_update(cluster_id)
        self._check_cluster_scale(cluster_id)

        job_exec_id = self._check_job_execution_create(cluster_id)
        self._check_job_execution_list(job_exec_id)
        self._check_job_execution_get(job_exec_id)
        self._check_job_execution_update(job_exec_id)

        self._check_job_execution_delete(job_exec_id)
        self._check_cluster_delete(cluster_id)
        self._check_deleting_tags(image_id)
        self._check_unregister_image(image_id)

    @classmethod
    def tearDownClass(cls):
        image_list = cls.client.images.list()
        image_id = cls.test_image_id
        if image_id in [image.id for image in image_list]:
            cls.client.images.unregister_image(image_id)
        super(JobExecutionTest, cls).tearDownClass()



# Copyright (c) 2013 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class BaseDataProcessingClient(rest_client.RestClient):

    def _request_and_check_resp(self, request_func, uri, resp_status):
        """Make a request and check response status code.

        It returns a ResponseBody.
        """
        resp, body = request_func(uri)
        self.expected_success(resp_status, resp.status)
        return rest_client.ResponseBody(resp, body)

    def _request_and_check_resp_data(self, request_func, uri, resp_status):
        """Make a request and check response status code.

        It returns pair: resp and response data.
        """
        resp, body = request_func(uri)
        self.expected_success(resp_status, resp.status)
        return resp, body

    def _request_check_and_parse_resp(self, request_func, uri,
                                      resp_status, *args, **kwargs):
        """Make a request, check response status code and parse response body.

        It returns a ResponseBody.
        """
        headers = {'Content-Type': 'application/json'}
        resp, body = request_func(uri, headers=headers, *args, **kwargs)
        self.expected_success(resp_status, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara_tests.scenario import base as base_scenario
from sahara_tests.scenario import utils


class CustomCheckKafka(object):
    def __init__(self, base_class):
        self.base = base_class

    def _run_command_on_node(self, *args, **kwargs):
        return self.base._run_command_on_node(*args, **kwargs)

    def _get_nodes_with_process(self, *args, **kwargs):
        return self.base._get_nodes_with_process(*args, **kwargs)

    def fail(self, *args, **kwargs):
        return self.base.fail(*args, **kwargs)

    def _prepare_job_running(self, *args, **kwargs):
        return self.base._prepare_job_running(*args, **kwargs)

    def _job_batching(self, *args, **kwargs):
        return self.base._job_batching(*args, **kwargs)

    @property
    def _results(self):
        return self.base._results

    @_results.setter
    def _results(self, value):
        self.base._results = value

    @staticmethod
    def _get_nodes_desc_list(nodes, node_domain, port):
        data = []
        for node in nodes:
            fqdn = "{0}.{1}".format(
                node["instance_name"], node_domain)
            data.append("{0}:{1}".format(fqdn, port))
        return ",".join(data)

    def _get_node_ip(self, process):
        node = self._get_nodes_with_process(process)[0]
        return node["management_ip"]

    def _search_file_on_node(self, ip, file):
        file_path = self._run_command_on_node(
            ip, 'find / -name "{file}" 2>/dev/null -print | head -n 1'
                .format(file=file))
        if not file_path:
            self.fail("Cannot find file: {file}".format(file))
        return file_path.rstrip()

    def _create_test_topic(self, broker, topic, zookeepers):
        ip = self._get_node_ip(broker)
        scr = self._search_file_on_node(ip, "kafka-topics.sh")
        # TODO(vgridnev): Avoid hardcoded values in future
        self._run_command_on_node(
            ip, "{script} --create --zookeeper {zoo} --replication-factor "
                "1 --partitions 1 --topic {topic}".format(
                    script=scr, zoo=zookeepers, topic=topic))

    def _send_messages(self, broker, topic, broker_list):
        ip = self._get_node_ip(broker)

        scr = self._search_file_on_node(ip, "kafka-console-producer.sh")
        messages = ["<<EOF", "banana", "in", "sahara", "sahara", "data",
                    "processing", "service", "stack", "open", "stack", "EOF"]
        cmd = "{script} --broker-list {brokers} --topic {topic} {msg}"
        self._run_command_on_node(
            ip, cmd.format(
                script=scr, topic=topic, brokers=broker_list,
                msg=" ".join(messages)))

    def _prepare_spark_kafka_job_running(self, shs):
        ip = self._get_node_ip(shs)
        utils_url = (
            "http://central.maven.org/maven2/org/apache/spark"
            "/spark-streaming-kafka-assembly_2.10/1.4.1"
            "/spark-streaming-kafka-assembly_2.10-1.4.1.jar")
        # try to search spark-kafka assembly utils
        result = self._search_file_on_node(ip, "spark-streaming-kafka")
        if not result:
            self._run_command_on_node(
                ip, "wget -P /tmp/spark-utils {url}".format(
                    url=utils_url))
        return self._search_file_on_node(ip, "spark-streaming-kafka")

    @base_scenario.track_result("Check Kafka", False)
    def check(self):
        # This check will check correct work of Kafka
        # Required things to run this check:
        # Cluster running with at least one ZooKeeper server and
        # Kafka Brokers and Spark can be included too
        # Initially designed for Ambari plugin.
        ckd = self.base.testcase.get(
            'custom_checks', {}).get('check_kafka', {})
        topic = ckd.get('topic', 'test-topic')
        topic = utils.rand_name(topic)
        zk = ckd.get('zookeeper_process', "ZooKeeper")
        kb = ckd.get('kafka_process', "Kafka Broker")
        shs = ckd.get('spark_process', "Spark History Server")
        # Disable spark job running by default
        spark_flow = ckd.get('spark_flow_test', None)
        kb_port = ckd.get('kafka_port', 6667)
        zk_port = ckd.get('zookeeper_port', 2181)
        node_domain = ckd.get('node_domain', "novalocal")
        broker_list = self._get_nodes_desc_list(
            self._get_nodes_with_process(kb), node_domain, kb_port)
        zookeeper_list = self._get_nodes_desc_list(
            self._get_nodes_with_process(zk), node_domain, zk_port)
        self._create_test_topic(kb, topic, zookeeper_list)
        self._send_messages(kb, topic, broker_list)
        if spark_flow:
            dest = self._prepare_spark_kafka_job_running(shs)
            if 'configs' not in spark_flow:
                spark_flow['configs'] = {}
            # override driver classpath
            spark_flow['configs']['edp.spark.driver.classpath'] = dest
            timeout = spark_flow.get('timeout', 30)
            if 'args' not in spark_flow:
                spark_flow['args'] = []
            new_args = []
            for arg in spark_flow['args']:
                arg = arg.format(zookeeper_list=zookeeper_list,
                                 timeout=timeout, topic=topic)
                new_args.append(arg)
            spark_flow['args'] = new_args
            to_execute = [self._prepare_job_running(spark_flow)]
            self._job_batching(to_execute)


def check(self):
    CustomCheckKafka(self).check()



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import functools
import logging
import os
import sys
import time
import traceback

import fixtures
from oslo_utils import timeutils
import prettytable
import six
from tempest.lib import base
from tempest.lib.common import ssh as connection
from tempest.lib import exceptions as exc

from sahara_tests.scenario import clients
from sahara_tests.scenario import timeouts
from sahara_tests.scenario import utils
from sahara_tests.utils import crypto as ssh
from sahara_tests.utils import url as utils_url

logger = logging.getLogger('swiftclient')
logger.setLevel(logging.CRITICAL)

CHECK_OK_STATUS = "OK"
CHECK_FAILED_STATUS = "FAILED"
CLUSTER_STATUS_ACTIVE = "Active"
CLUSTER_STATUS_ERROR = "Error"
HEALTH_CHECKS = ["RED", "YELLOW", "GREEN"]


def track_result(check_name, exit_with_error=True):
    def decorator(fct):
        @functools.wraps(fct)
        def wrapper(self, *args, **kwargs):
            started_at = timeutils.utcnow()
            test_info = {
                'check_name': check_name,
                'status': CHECK_OK_STATUS,
                'start_time': started_at,
                'duration': None,
                'traceback': None,
                'exception_time': None
            }
            self._results.append(test_info)
            try:
                return fct(self, *args, **kwargs)
            except Exception:
                test_info['exception_time'] = timeutils.utcnow().strftime(
                    '%Y%m%d_%H%M%S')
                test_info['status'] = CHECK_FAILED_STATUS
                test_info['traceback'] = traceback.format_exception(
                    *sys.exc_info())
                if exit_with_error:
                    raise
            finally:
                test_time = timeutils.utcnow() - started_at
                test_info['duration'] = test_time.seconds
        return wrapper
    return decorator


class BaseTestCase(base.BaseTestCase):

    @classmethod
    def setUpClass(cls):
        super(BaseTestCase, cls).setUpClass()
        cls.network = None
        cls.credentials = None
        cls.testcase = None
        cls._results = []
        cls.report = False
        cls.results_dir = '.'
        cls.default_templ_dir = '.'
        cls.use_api_v2 = False

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self._init_clients()
        timeouts.Defaults.init_defaults(self.testcase)
        self.testcase['ssh_username'] = self.sahara.register_image(
            self.glance.get_image_id(self.testcase['image']),
            self.testcase).username
        self.key = self.testcase.get('key_name')
        if self.key is None:
            self.private_key, self.public_key = ssh.generate_key_pair()
            self.key_name = self.__create_keypair()
        # save the private key if retain_resources is specified
        # (useful for debugging purposes)
        if self.testcase['retain_resources'] or self.key is None:
            private_key_file_name = os.path.join(self.results_dir,
                                                 self.key_name + '.key')
            with open(private_key_file_name, 'w+') as private_key_file:
                private_key_file.write(self.private_key)
            os.chmod(private_key_file_name, 0o600)
        self.plugin_version_option = 'plugin_version'
        if not self.use_api_v2:
            self.plugin_version_option = 'hadoop_version'
        self.plugin_opts = {
            'plugin_name': self.testcase['plugin_name'],
            self.plugin_version_option: self.testcase['plugin_version']
        }
        self.cinder = True
        self.proxy = False

    def _init_clients(self):
        username = self.credentials['os_username']
        password = self.credentials['os_password']
        tenant_name = self.credentials['os_tenant']
        auth_url = self.credentials['os_auth_url']
        sahara_service_type = self.credentials['sahara_service_type']
        sahara_url = self.credentials['sahara_url']
        auth_version = '3.0' if 'v3' in auth_url else '2.0'
        session = clients.get_session(auth_url, username, password,
                                      tenant_name,
                                      self.credentials.get('ssl_verify',
                                                           False),
                                      self._get_file_with_defaults(
                                          self.credentials.get('ssl_cert')))

        api_version = '2' if self.use_api_v2 else '1.1'
        self.sahara = clients.SaharaClient(session=session,
                                           service_type=sahara_service_type,
                                           sahara_url=sahara_url,
                                           api_version=api_version)
        self.nova = clients.NovaClient(session=session)
        self.neutron = clients.NeutronClient(session=session)
        # swiftclient doesn't support keystone sessions
        self.swift = clients.SwiftClient(
            auth_version=auth_version,
            authurl=auth_url,
            user=username,
            key=password,
            insecure=not self.credentials.get('ssl_verify', False),
            cacert=self.credentials.get('ssl_cert'),
            tenant_name=tenant_name)
        self.glance = clients.GlanceClient(session=session)
        # boto is not an OpenStack client, but we can handle it as well
        self.boto = None
        if self.credentials.get("s3_endpoint", None):
            self.boto = clients.BotoClient(
                endpoint=self.credentials["s3_endpoint"],
                accesskey=self.credentials["s3_accesskey"],
                secretkey=self.credentials["s3_secretkey"])

    def create_cluster(self):
        self.cluster_id = self.sahara.get_cluster_id(
            self.testcase.get('existing_cluster'))
        self.ng_id_map = {}
        if self.cluster_id is None:
            self.ng_id_map = self._create_node_group_templates()
            cl_tmpl_id = self._create_cluster_template()
            self.cluster_id = self._create_cluster(cl_tmpl_id)
        elif self.key is None:
            self.cinder = False
        self._poll_cluster_status_tracked(self.cluster_id)
        cluster = self.sahara.get_cluster(self.cluster_id, show_progress=True)
        self._get_proxy(cluster)
        self.check_cinder()
        if self.check_feature_available("provision_progress"):
            self._check_event_logs(cluster)

    def _get_proxy(self, cluster):
        for ng in cluster.node_groups:
            if ng['is_proxy_gateway']:
                for instance in ng['instances']:
                    if instance['management_ip'] != (
                            instance['internal_ip']):
                        self.proxy = instance['management_ip']

    @track_result("Check transient")
    def check_transient(self):
        with fixtures.Timeout(
                timeouts.Defaults.instance.timeout_check_transient,
                gentle=True):
            while True:
                if self.sahara.is_resource_deleted(
                        self.sahara.get_cluster_status, self.cluster_id):
                    break
                time.sleep(5)

    def _inject_datasources_data(self, arg, input_url, output_url):
        return arg.format(
            input_datasource=input_url, output_datasource=output_url)

    def _put_io_data_to_configs(self, configs, input_id, output_id):
        input_url, output_url = None, None
        if input_id is not None:
            input_url = self.sahara.get_datasource(
                data_source_id=input_id).url
        if output_id is not None:
            output_url = self.sahara.get_datasource(
                data_source_id=output_id).url
        pl = lambda x: self._inject_datasources_data(x, input_url, output_url)
        args = list(map(pl, configs.get('args', [])))
        configs['args'] = args
        return configs

    def _prepare_job_running(self, job):
        input_id, output_id = self._create_datasources(job)
        main_libs, additional_libs = self._create_job_binaries(job)
        job_id = self._create_job(job['type'], main_libs, additional_libs)
        configs = self._parse_job_configs(job)
        configs = self._put_io_data_to_configs(
            configs, input_id, output_id)
        return [job_id, input_id, output_id, configs]

    @track_result("Check EDP jobs", False)
    def check_run_jobs(self):
        batching = self.testcase.get('edp_batching',
                                     len(self.testcase['edp_jobs_flow']))
        batching_size = batching
        jobs = self.testcase.get('edp_jobs_flow', [])

        pre_exec = []
        for job in jobs:
            pre_exec.append(self._prepare_job_running(job))
            batching -= 1
            if not batching:
                self._job_batching(pre_exec)
                pre_exec = []
                batching = batching_size
        self.check_verification(self.cluster_id)

    def _job_batching(self, pre_exec):
        job_exec_ids = []
        for job_exec in pre_exec:
            job_exec_ids.append(self._run_job(*job_exec))

        self._poll_jobs_status(job_exec_ids)

    def _create_datasources(self, job):
        def create(ds, name):
            credential_vars = {}
            source = ds.get('source', None)
            destination = None if source else utils.rand_name(
                ds['destination'])
            if ds['type'] == 'swift':
                url = self._create_swift_data(source, destination)
                credential_vars = {
                    'credential_user': self.credentials['os_username'],
                    'credential_pass': self.credentials['os_password']
                }
            elif ds['type'] == 's3':
                url = self._create_s3_data(source, destination)
                credential_vars = {
                    's3_credentials': {
                        'accesskey': self.credentials['s3_accesskey'],
                        'secretkey': self.credentials['s3_secretkey'],
                        'endpoint': utils_url.url_schema_remover(
                            self.credentials['s3_endpoint']),
                        'ssl': self.credentials['s3_endpoint_ssl'],
                        'bucket_in_path': self.credentials['s3_bucket_path']
                    }
                }
            elif ds['type'] == 'hdfs':
                url = self._create_dfs_data(source, destination,
                                            self.testcase.get('hdfs_username',
                                                              'hadoop'),
                                            ds['type'])
            elif ds['type'] == 'maprfs':
                url = self._create_dfs_data(source, destination,
                                            ds.get('maprfs_username', 'mapr'),
                                            ds['type'])
            return self.__create_datasource(
                name=utils.rand_name(name),
                description='',
                data_source_type=ds['type'], url=url,
                **credential_vars)

        input_id, output_id = None, None
        if job.get('input_datasource'):
            ds = job['input_datasource']
            input_id = create(ds, 'input')

        if job.get('output_datasource'):
            ds = job['output_datasource']
            output_id = create(ds, 'output')

        return input_id, output_id

    def _create_job_binaries(self, job):
        main_libs = []
        additional_libs = []
        if job.get('main_lib'):
            main_libs.append(self._create_job_binary(job['main_lib']))
        for add_lib in job.get('additional_libs', []):
            lib_id = self._create_job_binary(add_lib)
            additional_libs.append(lib_id)

        return main_libs, additional_libs

    def _create_job_binary(self, job_binary):
        url = None
        extra = {}
        if job_binary['type'] == 'swift':
            url = self._create_swift_data(job_binary['source'])
            extra['user'] = self.credentials['os_username']
            extra['password'] = self.credentials['os_password']
        elif job_binary['type'] == 's3':
            url = self._create_s3_data(job_binary['source'])
            extra['accesskey'] = self.credentials['s3_accesskey']
            extra['secretkey'] = self.credentials['s3_secretkey']
            extra['endpoint'] = self.credentials['s3_endpoint']
        elif job_binary['type'] == 'database':
            url = self._create_internal_db_data(job_binary['source'])

        job_binary_name = '%s-%s' % (
            utils.rand_name('test'), os.path.basename(job_binary['source']))
        return self.__create_job_binary(job_binary_name, url, '', extra)

    def _create_job(self, type, mains, libs):
        return self.__create_job(utils.rand_name('test'), type, mains,
                                 libs, '')

    def _parse_job_configs(self, job):
        configs = {}
        if job.get('configs'):
            configs['configs'] = {}
            for param, value in six.iteritems(job['configs']):
                configs['configs'][param] = str(value)
        if job.get('args'):
            configs['args'] = list(map(str, job['args']))
        return configs

    def _run_job(self, job_id, input_id, output_id, configs):
        return self.__run_job(job_id, self.cluster_id, input_id, output_id,
                              configs)

    def _poll_jobs_status(self, exec_ids):
        try:
            with fixtures.Timeout(
                    timeouts.Defaults.instance.timeout_poll_jobs_status,
                    gentle=True):
                success = False
                polling_ids = list(exec_ids)
                while not success:
                    current_ids = list(polling_ids)
                    success = True
                    for exec_id in polling_ids:
                        status = self.sahara.get_job_status(exec_id)
                        if status not in ['FAILED', 'KILLED', 'DONEWITHERROR',
                                          "SUCCEEDED"]:
                            success = False
                        else:
                            current_ids.remove(exec_id)
                    polling_ids = list(current_ids)
                    time.sleep(5)
        finally:
            report = []
            for exec_id in exec_ids:
                status = self.sahara.get_job_status(exec_id)
                if status != "SUCCEEDED":
                    info = self.sahara.get_job_info(exec_id)
                    report.append("Job with id={id}, name={name}, "
                                  "type={type} has status "
                                  "{status}".format(id=exec_id,
                                                    name=info.name,
                                                    type=info.type,
                                                    status=status))
            if report:
                self.fail("\n".join(report))

    def _get_file_with_defaults(self, file_path):
        """ Check if the file exists; if it is a relative path, check also
            among the default files.
        """
        if not file_path:
            return ''

        all_files = [file_path]
        if not os.path.isabs(file_path):
            # relative path: look into default templates too, if defined
            default_file = os.path.join(self.default_templ_dir, file_path)
            if os.path.abspath(default_file) != os.path.abspath(file_path):
                all_files.append(default_file)
        for checked_file in all_files:
            if os.path.isfile(checked_file):
                return checked_file
        raise Exception('File %s not found while looking into %s' %
                        (file_path, all_files))

    def _read_source_file(self, source):
        if not source:
            return None
        with open(self._get_file_with_defaults(source), 'rb') as source_fd:
            data = source_fd.read()
        return data

    def _create_swift_data(self, source=None, destination=None):
        container = self._get_swift_container()
        path = utils.rand_name(destination if destination else 'test')
        data = self._read_source_file(source)

        self.__upload_to_container(container, path, data)

        return 'swift://%s.sahara/%s' % (container, path)

    def _create_s3_data(self, source=None, destination=None):
        bucket = self._get_s3_bucket()
        path = utils.rand_name(destination if destination else 'test')
        data = self._read_source_file(source)

        self.__upload_to_bucket(bucket, path, data)

        return 's3://%s/%s' % (bucket, path)

    def _create_dfs_data(self, source, destination, hdfs_username, fs):

        def to_hex_present(string):
            return "".join(map(lambda x: hex(ord(x)).replace("0x", "\\x"),
                               string.decode('utf-8')))
        if destination:
            return destination

        command_prefixes = {'hdfs': 'hdfs dfs',
                            'maprfs': 'hadoop fs'}

        hdfs_dir = utils.rand_name("/user/%s/data" % hdfs_username)
        instances = self._get_nodes_with_process('namenode')
        if len(instances) == 0:
            instances = self._get_nodes_with_process('CLDB')
        inst_ip = instances[0]["management_ip"]
        self._run_command_on_node(
            inst_ip,
            "sudo su - -c \"%(prefix)s -mkdir -p %(path)s \" %(user)s" % {
                "prefix": command_prefixes[fs],
                "path": hdfs_dir,
                "user": hdfs_username})
        hdfs_filepath = utils.rand_name(hdfs_dir + "/file")
        data = self._read_source_file(source)
        if not data:
            data = ''
        self._run_command_on_node(
            inst_ip,
            ("echo -e \"%(data)s\" | sudo su - -c \"%(prefix)s"
             " -put - %(path)s\" %(user)s") % {
                "data": to_hex_present(data),
                "prefix": command_prefixes[fs],
                "path": hdfs_filepath,
                "user": hdfs_username})
        return hdfs_filepath

    def _create_internal_db_data(self, source):
        data = self._read_source_file(source)
        id = self.__create_internal_db_data(utils.rand_name('test'), data)
        return 'internal-db://%s' % id

    def _get_swift_container(self):
        if not getattr(self, '__swift_container', None):
            self.__swift_container = self.__create_container(
                utils.rand_name('sahara-tests'))
        return self.__swift_container

    def _get_s3_bucket(self):
        if not getattr(self, '__s3_bucket', None):
            self.__s3_bucket = self.__create_bucket(
                utils.rand_name('sahara-tests'))
        return self.__s3_bucket

    @track_result("Cluster scaling", False)
    def check_scale(self):
        scale_ops = []
        ng_before_scale = self.sahara.get_cluster(self.cluster_id).node_groups
        scale_ops = self.testcase['scaling']

        body = {}
        for op in scale_ops:
            node_scale = op['node_group']
            if op['operation'] == 'add':
                if 'add_node_groups' not in body:
                    body['add_node_groups'] = []
                body['add_node_groups'].append({
                    'node_group_template_id':
                    self.ng_id_map.get(node_scale,
                                       self.sahara.get_node_group_template_id(
                                           node_scale)),
                    'count': op['size'],
                    'name': utils.rand_name(node_scale)
                })
            if op['operation'] == 'resize':
                if 'resize_node_groups' not in body:
                    body['resize_node_groups'] = []
                body['resize_node_groups'].append({
                    'name': self.ng_name_map.get(
                        node_scale,
                        self.sahara.get_node_group_template_id(node_scale)),
                    'count': op['size']
                })

        if body:
            self.sahara.scale_cluster(self.cluster_id, body)
            self._poll_cluster_status(self.cluster_id)
            ng_after_scale = self.sahara.get_cluster(
                self.cluster_id).node_groups
            self._validate_scaling(ng_after_scale,
                                   self._get_expected_count_of_nodes(
                                       ng_before_scale, body))

    def _validate_scaling(self, after, expected_count):
        for (key, value) in six.iteritems(expected_count):
            ng = {}
            for after_ng in after:
                if after_ng['name'] == key:
                    ng = after_ng
                    break
            self.assertEqual(value, ng.get('count', 0))

    def _get_expected_count_of_nodes(self, before, body):
        expected_mapper = {}
        for ng in before:
            expected_mapper[ng['name']] = ng['count']
        for ng in body.get('add_node_groups', []):
            expected_mapper[ng['name']] = ng['count']
        for ng in body.get('resize_node_groups', []):
            expected_mapper[ng['name']] = ng['count']
        return expected_mapper

    @track_result("Check cinder volumes")
    def check_cinder(self):
        if not self._get_node_list_with_volumes() or not self.cinder:
            print("All tests for Cinder were skipped")
            return
        for node_with_volumes in self._get_node_list_with_volumes():
            volume_count_on_node = int(self._run_command_on_node(
                node_with_volumes['node_ip'],
                'mount | grep %s | wc -l' %
                node_with_volumes['volume_mount_prefix']
            ))
            self.assertEqual(
                node_with_volumes['volume_count'], volume_count_on_node,
                'Some volumes were not mounted to node.\n'
                'Expected count of mounted volumes to node is %s.\n'
                'Actual count of mounted volumes to node is %s.'
                % (node_with_volumes['volume_count'], volume_count_on_node)
            )

    def _get_node_list_with_volumes(self):
        node_groups = self.sahara.get_cluster(self.cluster_id).node_groups
        node_list_with_volumes = []
        for node_group in node_groups:
            if node_group['volumes_per_node'] != 0:
                for instance in node_group['instances']:
                    node_list_with_volumes.append({
                        'node_ip': instance['management_ip'],
                        'volume_count': node_group['volumes_per_node'],
                        'volume_mount_prefix':
                            node_group['volume_mount_prefix']
                    })
        return node_list_with_volumes

    @track_result("Create node group templates")
    def _create_node_group_templates(self):
        ng_id_map = {}
        floating_ip_pool = None
        security_group = None
        proxy_exist = False

        if self.network['public_network']:
            if self.network['type'] == 'neutron':
                floating_ip_pool = self.neutron.get_network_id(
                    self.network['public_network'])
            elif not self.network['auto_assignment_floating_ip']:
                floating_ip_pool = self.network['public_network']

        node_groups = []
        for ng in self.testcase['node_group_templates']:
            node_groups.append(ng)
            if ng.get('is_proxy_gateway', False):
                proxy_exist = True

        for ng in node_groups:
            kwargs = dict(ng)
            kwargs.update(self.plugin_opts)
            kwargs['flavor_id'] = self._get_flavor_id(kwargs['flavor'])
            del kwargs['flavor']
            kwargs['name'] = utils.rand_name(kwargs['name'])
            if (not proxy_exist) or (proxy_exist and kwargs.get(
                    'is_proxy_gateway', False)):
                kwargs['floating_ip_pool'] = floating_ip_pool
            if not kwargs.get('auto_security_group', True):
                if security_group is None:
                    sg_name = utils.rand_name('scenario')
                    security_group = self.__create_security_group(sg_name)
                    self.neutron.add_security_group_rule_for_neutron(
                        security_group)
                kwargs['security_groups'] = [security_group]

            # boot_from_volume requires APIv2
            if kwargs.get('boot_from_volume', False) and not self.use_api_v2:
                raise Exception('boot_from_volume is set for %s but it '
                                'requires APIv2' % (kwargs['name']))

            ng_id = self.__create_node_group_template(**kwargs)
            ng_id_map[ng['name']] = ng_id
        return ng_id_map

    @track_result("Set flavor")
    def _get_flavor_id(self, flavor):
        if isinstance(flavor, six.string_types):
            return self.nova.get_flavor_id(flavor)
        else:
            # if the name already exists, use it
            if flavor.get('name'):
                try:
                    return self.nova.get_flavor_id(flavor['name'])
                except exc.NotFound:
                    print("Custom flavor %s not found, it will be created" %
                          (flavor['name']))

            flavor_id = self.nova.create_flavor(flavor).id
            self.addCleanup(self.nova.delete_flavor, flavor_id)
            return flavor_id

    @track_result("Create cluster template")
    def _create_cluster_template(self):
        self.ng_name_map = {}
        template = self.testcase['cluster_template']

        kwargs = dict(template)
        ngs = kwargs['node_group_templates']
        del kwargs['node_group_templates']
        kwargs['node_groups'] = []
        for ng, count in ngs.items():
            ng_name = utils.rand_name(ng)
            self.ng_name_map[ng] = ng_name
            kwargs['node_groups'].append({
                'name': ng_name,
                'node_group_template_id': self.ng_id_map[ng],
                'count': count})

        kwargs.update(self.plugin_opts)
        kwargs['name'] = utils.rand_name(kwargs.get('name', 'ct'))
        if self.network['type'] == 'neutron':
            kwargs['net_id'] = self.neutron.get_network_id(
                self.network['private_network'])

        return self.__create_cluster_template(**kwargs)

    @track_result("Check event logs")
    def _check_event_logs(self, cluster):
        invalid_steps = []
        if cluster.is_transient:
            # skip event log testing
            return

        for step in cluster.provision_progress:
            if not step['successful']:
                invalid_steps.append(step)

        if len(invalid_steps) > 0:
            invalid_steps_info = "\n".join(six.text_type(e)
                                           for e in invalid_steps)
            steps_info = "\n".join(six.text_type(e)
                                   for e in cluster.provision_progress)
            raise exc.TempestException(
                "Issues with event log work: "
                "\n Incomplete steps: \n\n {invalid_steps}"
                "\n All steps: \n\n {steps}".format(
                    steps=steps_info,
                    invalid_steps=invalid_steps_info))

    @track_result("Create cluster")
    def _create_cluster(self, cluster_template_id):
        if self.testcase.get('cluster'):
            kwargs = dict(self.testcase['cluster'])
        else:
            kwargs = {}  # default template

        kwargs.update(self.plugin_opts)
        kwargs['name'] = utils.rand_name(kwargs.get('name', 'test'))
        kwargs['cluster_template_id'] = cluster_template_id
        kwargs['default_image_id'] = self.glance.get_image_id(
            self.testcase['image'])
        kwargs['user_keypair_id'] = self.key_name

        return self.__create_cluster(**kwargs)

    @track_result("Check cluster state")
    def _poll_cluster_status_tracked(self, cluster_id):
        self._poll_cluster_status(cluster_id)

    def _poll_cluster_status(self, cluster_id):
        with fixtures.Timeout(
                timeouts.Defaults.instance.timeout_poll_cluster_status,
                gentle=True):
            while True:
                status = self.sahara.get_cluster_status(cluster_id)
                if status == CLUSTER_STATUS_ACTIVE:
                    break
                if status == CLUSTER_STATUS_ERROR:
                    cluster = self.sahara.get_cluster(cluster_id)
                    failure_desc = cluster.status_description
                    message = ("Cluster in %s state with"
                               " a message below:\n%s") % (status,
                                                           failure_desc)

                    raise exc.TempestException(message)
                time.sleep(3)

    def _run_command_on_node(self, node_ip, command):
        host_ip = node_ip
        if self.proxy:
            host_ip = self.proxy
            command = ("echo '{pkey}' > {filename} && chmod 600 {filename} && "
                       "ssh -o StrictHostKeyChecking=no {ip} -i {filename} "
                       "'{cmd}' && rm {filename}".format(
                           pkey=self.private_key, filename='scenario.pem',
                           ip=node_ip, cmd=command))
        ssh_session = connection.Client(host_ip, self.testcase['ssh_username'],
                                        pkey=self.private_key)
        return ssh_session.exec_command(command)

    def _get_nodes_with_process(self, process=None):
        if process is not None:
            process = process.lower()
        nodegroups = self.sahara.get_cluster(self.cluster_id).node_groups
        nodes_with_process = []
        for nodegroup in nodegroups:
            for node_process in nodegroup['node_processes']:
                if not process or process in node_process.lower():
                    nodes_with_process.extend(nodegroup['instances'])
        return nodes_with_process

    def _get_health_status(self, cluster):
        try:
            return cluster.verification['status']
        except (AttributeError, KeyError):
            return 'UNKNOWN'

    def _poll_verification_status(self, cluster_id):
        with fixtures.Timeout(
                timeouts.Defaults.instance.timeout_poll_cluster_status,
                gentle=True):
            while True:
                cluster = self.sahara.get_cluster(cluster_id)
                status = self._get_health_status(cluster)
                if status == 'UNKNOWN':
                    print("Cluster verification did not start")
                    break
                if status in HEALTH_CHECKS:
                    break
                time.sleep(3)

    @track_result("Check cluster verification")
    def check_verification(self, cluster_id):
        if self.check_feature_available("verification"):
            self._poll_cluster_status(cluster_id)

            # need to check if previous verification check is not
            # in the status CHECKING
            self._poll_verification_status(cluster_id)
            self.sahara.start_cluster_verification(cluster_id)

            # check if this verification check finished without errors
            self._poll_verification_status(cluster_id)
        else:
            print("All tests for cluster verification were skipped")

    # client ops

    def __create_node_group_template(self, *args, **kwargs):
        id = self.sahara.create_node_group_template(*args, **kwargs)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.sahara.delete_node_group_template, id)
        return id

    def __create_security_group(self, sg_name):
        id = self.neutron.create_security_group_for_neutron(sg_name)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.neutron.delete_security_group_for_neutron, id)
        return id

    def __create_cluster_template(self, *args, **kwargs):
        id = self.sahara.create_cluster_template(*args, **kwargs)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.sahara.delete_cluster_template, id)
        return id

    def __create_cluster(self, *args, **kwargs):
        id = self.sahara.create_cluster(*args, **kwargs)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.sahara.delete_cluster, id)
        return id

    def __create_datasource(self, *args, **kwargs):
        id = self.sahara.create_datasource(*args, **kwargs)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.sahara.delete_datasource, id)
        return id

    def __create_internal_db_data(self, *args, **kwargs):
        id = self.sahara.create_job_binary_internal(*args, **kwargs)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.sahara.delete_job_binary_internal, id)
        return id

    def __create_job_binary(self, *args, **kwargs):
        id = self.sahara.create_job_binary(*args, **kwargs)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.sahara.delete_job_binary, id)
        return id

    def __create_job(self, *args, **kwargs):
        id = self.sahara.create_job_template(*args, **kwargs)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.sahara.delete_job_template, id)
        return id

    def __run_job(self, *args, **kwargs):
        id = self.sahara.run_job(*args, **kwargs)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.sahara.delete_job_execution, id)
        return id

    def __create_container(self, container_name):
        self.swift.create_container(container_name)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.swift.delete_container, container_name)
        return container_name

    def __upload_to_container(self, container_name, object_name, data=None):
        if data:
            self.swift.upload_data(container_name, object_name, data)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.swift.delete_object, container_name,
                            object_name)

    def __create_bucket(self, bucket_name):
        self.boto.create_bucket(bucket_name)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.boto.delete_bucket, bucket_name)
        return bucket_name

    def __upload_to_bucket(self, bucket_name, object_name, data=None):
        if data:
            self.boto.upload_data(bucket_name, object_name, data)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.boto.delete_object, bucket_name,
                            object_name)

    def __create_keypair(self):
        key = utils.rand_name('scenario_key')
        self.nova.nova_client.keypairs.create(key,
                                              public_key=self.public_key)
        if not self.testcase['retain_resources']:
            self.addCleanup(self.nova.delete_keypair, key)
        return key

    def check_feature_available(self, feature_name):
        if not getattr(self.sahara.get_cluster(self.cluster_id),
                       feature_name, None):
            return False
        return True

    def tearDown(self):
        tbs = []
        table = prettytable.PrettyTable(["Check", "Status", "Duration, s",
                                         "Start time"])
        table.align["Check"] = "l"
        for check in self._results:
            table.add_row(
                [check['check_name'], check['status'], check['duration'],
                 check['start_time']])
            if check['status'] == CHECK_FAILED_STATUS:
                tbs.append(check['exception_time'])
                tbs.extend(check['traceback'])
                tbs.append("")
        print("Results of testing plugin", self.plugin_opts['plugin_name'],
              self.plugin_opts[self.plugin_version_option])
        print(table)
        print("\n".join(tbs), file=sys.stderr)

        super(BaseTestCase, self).tearDown()

        test_failed = any([c['status'] == CHECK_FAILED_STATUS
                           for c in self._results])

        if self.report:
            filename = {"time": time.strftime('%Y%m%d%H%M%S',
                                              time.localtime())}
            filename.update(self.plugin_opts)
            # let's normalize this variable so that we can use
            # a stable name as formatter later.
            if 'hadoop_version' in filename:
                filename['plugin_version'] = filename['hadoop_version']
                del filename['hadoop_version']
            report_file_name = os.path.join(
                self.results_dir,
                '{plugin_name}_{plugin_version}-{time}'.format(**filename))
            time.strftime('%Y%m%d%H%M%S', time.localtime())
            with open(report_file_name, 'w+') as report_file:
                report_file.write(str(self._results))
            print("Results can be found in %s" % report_file_name)
        if test_failed:
            self.fail("Scenario tests failed")



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

service_option = cfg.BoolOpt("sahara",
                             default=False,
                             help="Whether or not sahara is expected to be "
                                  "available")

data_processing_group = cfg.OptGroup(name="data-processing",
                                     title="Data Processing options")

DataProcessingGroup = [
    cfg.StrOpt('catalog_type',
               default='data-processing',
               help="Catalog type of the data processing service."),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               choices=['public', 'admin', 'internal',
                        'publicURL', 'adminURL', 'internalURL'],
               help="The endpoint type to use for the data processing "
                    "service."),
]

DataProcessingAdditionalGroup = [
    cfg.IntOpt('cluster_timeout',
               default=3600,
               help='Timeout (in seconds) to wait for cluster deployment.'),
    cfg.IntOpt('request_timeout',
               default=10,
               help='Timeout (in seconds) between status checks.'),
    # FIXME: the default values here are an hack needed until it is possible
    # to pass values from the job to tempest.conf (or a devstack plugin is
    # written).
    cfg.StrOpt('test_image_name',
               default='xenial-server-cloudimg-amd64-disk1',
               help='name of an image which is used for cluster creation.'),
    cfg.StrOpt('test_ssh_user',
               default='ubuntu',
               help='username used to access the test image.'),
    cfg.BoolOpt('use_api_v2',
                default=False,
                help='Run API tests against APIv2 instead of 1.1'),
    cfg.StrOpt('api_version_saharaclient',
               default='1.1',
               help='Version of Sahara API used by saharaclient',
               deprecated_name='saharaclient_version'),
    cfg.StrOpt('sahara_url',
               help='Sahara url as http://ip:port/api_version/tenant_id'),
    # TODO(shuyingya): Delete this option once the Mitaka release is EOL.
    cfg.BoolOpt('plugin_update_support',
                default=True,
                help='Does sahara support plugin update?'),
]


data_processing_feature_group = cfg.OptGroup(
    name="data-processing-feature-enabled",
    title="Enabled Data Processing features")

DataProcessingFeaturesGroup = [
    cfg.ListOpt('plugins',
                default=["vanilla", "cdh"],
                help="List of enabled data processing plugins"),
    # delete this and always execute the tests when Tempest and
    # this Tempest plugin stop supporting Queens, the last version
    # without or incomplete S3 support.
    cfg.BoolOpt('s3',
                default=False,
                help='Does Sahara support S3?'),
]



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sahara_tempest_plugin.tests.cli import base


class SaharaImageCLITest(base.ClientTestBase):

    def openstack_image_list(self):
        self.assertTableStruct(self.listing_result('image list'), [
            'Name',
            'Id',
            'Username',
            'Tags'
        ])

    def openstack_image_register(self, name_to_register, username):
        images_list = self.openstack('image list')
        images = self.parser.listing(images_list)
        images_name = [p['Name'] for p in images]
        flag = None
        for image_name in images_name:
            if image_name == name_to_register:
                flag = (' --username %s %s' % (username, image_name))
        if flag is None:
            raise self.skipException('No available image for testing')
        self.assertTableStruct(
            self.listing_result(''.join(['image register', flag])), [
                'Field',
                'Value'
            ])
        return name_to_register

    def openstack_image_show(self, image_name):
        self.assertTableStruct(
            self.listing_result(''.join(['image show ', image_name])), [
                'Field',
                'Value'
            ])

    def openstack_image_tags_add(self, image_name):
        plugin = self.get_default_plugin()
        flag = '%s --tags %s %s' % (image_name,
                                    plugin['Name'],
                                    plugin['Versions'])
        self.assertTableStruct(
            self.listing_result(''.join(['image tags add ', flag])), [
                'Field',
                'Value'
            ])

    def openstack_image_tags_set(self, image_name):
        flag = ''.join([image_name, ' --tags update_tag'])
        self.assertTableStruct(
            self.listing_result(''.join(['image tags set ', flag])), [
                'Field',
                'Value'
            ])

    def openstack_image_tags_remove(self, image_name):
        flag = ''.join([image_name, ' --tags update_tag'])
        self.assertTableStruct(
            self.listing_result(''.join(['image tags remove ', flag])), [
                'Field',
                'Value'
            ])

    def openstack_image_unregister(self, image_name):
        self.assertTableStruct(
            self.listing_result(''.join(['image unregister ', image_name])), [
                'Field',
                'Value'
            ])

    def negative_unregister_not_existing_image(self, image_name):
        """Test to unregister already unregistrated image"""
        command_to_execute = 'image unregister'
        self.check_negative_scenarios(base.TEMPEST_ERROR_MESSAGE,
                                      command_to_execute,
                                      image_name)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import time

import fixtures
from botocore.exceptions import ClientError as BotoClientError
from botocore import session as botocore_session
from glanceclient import client as glance_client
from keystoneauth1.identity import v3 as identity_v3
from keystoneauth1 import session
from neutronclient.neutron import client as neutron_client
from novaclient import client as nova_client
from novaclient import exceptions as nova_exc
from oslo_utils import uuidutils
from saharaclient.api import base as saharaclient_base
from saharaclient import client as sahara_client
import six
from swiftclient import client as swift_client
from swiftclient import exceptions as swift_exc
from tempest.lib import exceptions as exc

from sahara_tests.scenario import timeouts
from sahara_tests.scenario import utils


def get_session(auth_url=None, username=None, password=None,
                project_name=None, verify=True, cert=None):
    auth_url_fixed = auth_url.replace('/v2.0', '/v3')
    if not auth_url_fixed.endswith('/v3'):
        auth_url_fixed += '/v3'
    auth = identity_v3.Password(auth_url=auth_url_fixed,
                                username=username,
                                password=password,
                                project_name=project_name,
                                user_domain_name='default',
                                project_domain_name='default')
    return session.Session(auth=auth, verify=verify, cert=cert)


class Client(object):
    def is_resource_deleted(self, method, *args, **kwargs):
        raise NotImplementedError

    def delete_resource(self, method, *args, **kwargs):
        with fixtures.Timeout(
                timeouts.Defaults.instance.timeout_delete_resource,
                gentle=True):
            while True:
                if self.is_resource_deleted(method, *args, **kwargs):
                    break
                time.sleep(5)


class SaharaClient(Client):
    def __init__(self, *args, **kwargs):
        self.api_version = '1.1'
        if 'api_version' in kwargs:
            self.api_version = kwargs['api_version']
            del kwargs['api_version']
        self.sahara_client = sahara_client.Client(self.api_version, *args,
                                                  **kwargs)

    def create_node_group_template(self, *args, **kwargs):
        data = self.sahara_client.node_group_templates.create(*args, **kwargs)
        return data.id

    def delete_node_group_template(self, node_group_template_id):
        return self.delete_resource(
            self.sahara_client.node_group_templates.delete,
            node_group_template_id)

    def create_cluster_template(self, *args, **kwargs):
        data = self.sahara_client.cluster_templates.create(*args, **kwargs)
        return data.id

    def delete_cluster_template(self, cluster_template_id):
        return self.delete_resource(
            self.sahara_client.cluster_templates.delete,
            cluster_template_id)

    def create_cluster(self, *args, **kwargs):
        data = self.sahara_client.clusters.create(*args, **kwargs)
        return data.id

    def delete_cluster(self, cluster_id):
        return self.delete_resource(
            self.sahara_client.clusters.delete,
            cluster_id)

    def scale_cluster(self, cluster_id, body):
        return self.sahara_client.clusters.scale(cluster_id, body)

    def start_cluster_verification(self, cluster_id):
        return self.sahara_client.clusters.verification_update(cluster_id,
                                                               'START')

    def create_datasource(self, *args, **kwargs):
        data = self.sahara_client.data_sources.create(*args, **kwargs)
        return data.id

    def get_datasource(self, *args, **kwargs):
        return self.sahara_client.data_sources.get(*args, **kwargs)

    def delete_datasource(self, datasource_id):
        return self.delete_resource(
            self.sahara_client.data_sources.delete,
            datasource_id)

    def create_job_binary_internal(self, *args, **kwargs):
        data = self.sahara_client.job_binary_internals.create(*args, **kwargs)
        return data.id

    def delete_job_binary_internal(self, job_binary_internal_id):
        return self.delete_resource(
            self.sahara_client.job_binary_internals.delete,
            job_binary_internal_id)

    def create_job_binary(self, *args, **kwargs):
        data = self.sahara_client.job_binaries.create(*args, **kwargs)
        return data.id

    def delete_job_binary(self, job_binary_id):
        return self.delete_resource(
            self.sahara_client.job_binaries.delete,
            job_binary_id)

    def create_job_template(self, *args, **kwargs):
        if self.api_version == '1.1':
            data = self.sahara_client.jobs.create(*args, **kwargs)
        else:
            data = self.sahara_client.job_templates.create(*args, **kwargs)
        return data.id

    def delete_job_template(self, job_id):
        if self.api_version == '1.1':
            delete_function = self.sahara_client.jobs.delete
        else:
            delete_function = self.sahara_client.job_templates.delete
        return self.delete_resource(delete_function, job_id)

    def run_job(self, *args, **kwargs):
        if self.api_version == '1.1':
            data = self.sahara_client.job_executions.create(*args, **kwargs)
        else:
            data = self.sahara_client.jobs.create(*args, **kwargs)
        return data.id

    def delete_job_execution(self, job_execution_id):
        if self.api_version == '1.1':
            delete_function = self.sahara_client.job_executions.delete
        else:
            delete_function = self.sahara_client.jobs.delete
        return self.delete_resource(delete_function, job_execution_id)

    def get_cluster(self, cluster_id, show_progress=False):
        return self.sahara_client.clusters.get(cluster_id, show_progress)

    def get_cluster_status(self, cluster_id):
        data = self.sahara_client.clusters.get(cluster_id)
        return str(data.status)

    def get_job_status(self, exec_id):
        if self.api_version == '1.1':
            data = self.sahara_client.job_executions.get(exec_id)
        else:
            data = self.sahara_client.jobs.get(exec_id)
        return str(data.info['status'])

    def get_job_info(self, exec_id):
        if self.api_version == '1.1':
            job_execution = self.sahara_client.job_executions.get(exec_id)
        else:
            job_execution = self.sahara_client.jobs.get(exec_id)
        return self.sahara_client.jobs.get(job_execution.job_id)

    def get_cluster_id(self, name):
        if uuidutils.is_uuid_like(name):
            return name
        for cluster in self.sahara_client.clusters.list():
            if cluster.name == name:
                return cluster.id

    def get_node_group_template_id(self, name):
        for nodegroup in self.sahara_client.node_group_templates.list():
            if nodegroup.name == name:
                return nodegroup.id

    def register_image(self, image_id, testcase):
        try:
            return self.sahara_client.images.get(image_id)
        except saharaclient_base.APIException:
            print("Image not registered in sahara. Registering and run tests")
            if testcase.get('image_username') is not None:
                self.sahara_client.images.update_image(
                    image_id, testcase.get('image_username'),
                    "Registered by scenario tests")
                self.sahara_client.images.update_tags(
                    image_id, [testcase["plugin_name"],
                               testcase["plugin_version"]])
            else:
                raise exc.InvalidContentType(
                    "Registering of image failed. Please, specify "
                    "'image_username'. For details see README in scenario "
                    "tests.")
        return self.sahara_client.images.get(image_id)

    def is_resource_deleted(self, method, *args, **kwargs):
        try:
            method(*args, **kwargs)
        except saharaclient_base.APIException as ex:
            return ex.error_code == 404

        return False


class NovaClient(Client):
    def __init__(self, *args, **kwargs):
        self.nova_client = nova_client.Client('2', *args, **kwargs)

    def get_flavor_id(self, flavor_name):
        if (uuidutils.is_uuid_like(flavor_name) or
                (isinstance(flavor_name, six.string_types) and
                 flavor_name.isdigit())):
            return flavor_name
        for flavor in self.nova_client.flavors.list():
            if flavor.name == flavor_name:
                return flavor.id

        raise exc.NotFound(flavor_name)

    def create_flavor(self, flavor_object):
        return self.nova_client.flavors.create(
            flavor_object.get('name', utils.rand_name('scenario')),
            flavor_object.get('ram', 1),
            flavor_object.get('vcpus', 1),
            flavor_object.get('root_disk', 0),
            ephemeral=flavor_object.get('ephemeral_disk', 0),
            swap=flavor_object.get('swap_disk', 0),
            flavorid=flavor_object.get('id', 'auto'))

    def delete_flavor(self, flavor_id):
        return self.delete_resource(self.nova_client.flavors.delete, flavor_id)

    def delete_keypair(self, key_name):
        return self.delete_resource(
            self.nova_client.keypairs.delete, key_name)

    def is_resource_deleted(self, method, *args, **kwargs):
        try:
            method(*args, **kwargs)
        except nova_exc.NotFound as ex:
            return ex.code == 404

        return False


class NeutronClient(Client):
    def __init__(self, *args, **kwargs):
        self.neutron_client = neutron_client.Client('2.0', *args, **kwargs)

    def get_network_id(self, network_name):
        if uuidutils.is_uuid_like(network_name):
            return network_name
        networks = self.neutron_client.list_networks(name=network_name)
        networks = networks['networks']
        if len(networks) < 1:
            raise exc.NotFound(network_name)
        return networks[0]['id']

    def create_security_group_for_neutron(self, sg_name):
        security_group = self.neutron_client.create_security_group({
            "security_group":
                {
                    "name": sg_name,
                    "description": "Just for test"
                }
        })
        return security_group['security_group']['id']

    def get_security_group_id(self, sg_name):
        for sg in (self.neutron_client.list_security_groups()
                   ["security_groups"]):
            if sg['name'] == sg_name:
                return sg['id']

        raise exc.NotFound(sg_name)

    def add_security_group_rule_for_neutron(self, sg_id):
        return self.neutron_client.create_security_group_rule({
            "security_group_rules": [
                {
                    "direction": "ingress",
                    "ethertype": "IPv4",
                    "port_range_max": 65535,
                    "port_range_min": 1,
                    "protocol": "TCP",
                    "remote_group_id": None,
                    "remote_ip_prefix": None,
                    "security_group_id": sg_id
                },
                {
                    "direction": "egress",
                    "ethertype": "IPv4",
                    "port_range_max": 65535,
                    "port_range_min": 1,
                    "protocol": "TCP",
                    "remote_group_id": None,
                    "remote_ip_prefix": None,
                    "security_group_id": sg_id
                }
            ]
        })

    def delete_security_group_for_neutron(self, sg_id):
        return self.neutron_client.delete_security_group(sg_id)


class SwiftClient(Client):
    def __init__(self, *args, **kwargs):
        self.swift_client = swift_client.Connection(*args, **kwargs)

    def create_container(self, container_name):
        return self.swift_client.put_container(container_name)

    def delete_container(self, container_name):
        objects = self._get_objects(container_name)
        for obj in objects:
            self.delete_object(container_name, obj)
        return self.delete_resource(
            self.swift_client.delete_container, container_name)

    def _get_objects(self, container_name):
        metadata = self.swift_client.get_container(container_name)
        objects = []
        for obj in metadata[1]:
            objects.append(obj['name'])
        return objects[::-1]

    def upload_data(self, container_name, object_name, data):
        return self.swift_client.put_object(container_name, object_name, data)

    def delete_object(self, container_name, object_name):
        return self.delete_resource(
            self.swift_client.delete_object,
            container_name,
            object_name)

    def is_resource_deleted(self, method, *args, **kwargs):
        try:
            method(*args, **kwargs)
        except swift_exc.ClientException as ex:
            return ex.http_status == 404

        return False


class BotoClient(Client):
    def __init__(self, *args, **kwargs):
        sess = botocore_session.get_session()
        self.boto_client = sess.create_client(
            's3',
            endpoint_url=kwargs['endpoint'],
            aws_access_key_id=kwargs['accesskey'],
            aws_secret_access_key=kwargs['secretkey']
        )

    def create_bucket(self, bucket_name):
        return self.boto_client.create_bucket(Bucket=bucket_name)

    def _delete_and_check_bucket(self, bucket_name):
        bucket_deleted = False
        operation_parameters = {'Bucket': bucket_name}
        try:
            # While list_objects_v2 is the suggested function, pagination
            # does not seems to work properly with RadosGW when it's used.
            paginator = self.boto_client.get_paginator('list_objects')
            page_iterator = paginator.paginate(**operation_parameters)
            for page in page_iterator:
                if 'Contents' not in page:
                    continue
                for item in page['Contents']:
                    self.boto_client.delete_object(Bucket=bucket_name,
                                                   Key=item['Key'])
            self.boto_client.delete_bucket(Bucket=bucket_name)
        except BotoClientError as ex:
            error = ex.response.get('Error', {})
            # without the conversion the value is a tuple
            error_code = '%s' % (error.get('Code', ''))
            if error_code == 'NoSuchBucket':
                bucket_deleted = True
        return bucket_deleted

    def delete_bucket(self, bucket_name):
        return self.delete_resource(
            self._delete_and_check_bucket, bucket_name)

    def upload_data(self, bucket_name, object_name, data):
        return self.boto_client.put_object(
            Bucket=bucket_name,
            Key=object_name,
            Body=data)

    def _delete_and_check_object(self, bucket_name, object_name):
        self.boto_client.delete_object(Bucket=bucket_name, Key=object_name)
        object_deleted = False
        try:
            self.boto_client.head_object(Bucket=bucket_name, Key=object_name)
        except BotoClientError as ex:
            error = ex.response.get('Error', {})
            # without the conversion the value is a tuple
            error_code = '%s' % (error.get('Code', ''))
            if error_code == '404':
                object_deleted = True
        return object_deleted

    def delete_object(self, bucket_name, object_name):
        return self.delete_resource(
            self._delete_and_check_object,
            bucket_name, object_name)

    def is_resource_deleted(self, method, *args, **kwargs):
        # Exceptions are handled directly inside the call to "method",
        # because they are not the same for objects and buckets.
        return method(*args, **kwargs)


class GlanceClient(Client):
    def __init__(self, *args, **kwargs):
        self.glance_client = glance_client.Client('2', *args, **kwargs)

    def get_image_id(self, image_name):
        if uuidutils.is_uuid_like(image_name):
            return image_name
        for image in self.glance_client.images.list():
            if image.name == image_name:
                return image.id
        raise exc.NotFound(image_name)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest import config
from tempest.lib import decorators

from sahara_tempest_plugin.tests.cli import clusters
from sahara_tempest_plugin.tests.cli import cluster_templates
from sahara_tempest_plugin.tests.cli import images
from sahara_tempest_plugin.tests.cli import node_group_templates
from sahara_tempest_plugin.tests.cli import plugins
from sahara_tempest_plugin.tests.cli import job_binaries
from sahara_tempest_plugin.tests.cli import jobs
from sahara_tempest_plugin.tests.cli import job_templates
from sahara_tempest_plugin.tests.cli import data_sources
from sahara_tempest_plugin.tests.cli import job_types

TEMPEST_CONF = config.CONF
NODE_GROUP_TEMPLATE = 'node group template'


class Scenario(images.SaharaImageCLITest,
               node_group_templates.SaharaNodeGroupCLITest,
               cluster_templates.SaharaClusterTemplateCLITest,
               clusters.SaharaClusterCLITest,
               plugins.SaharaPluginCLITest,
               job_binaries.SaharaJobBinaryCLITest,
               jobs.SaharaJobCLITest,
               job_templates.SaharaJobTemplateCLITest,
               data_sources.SaharaDataSourceCLITest,
               job_types.SaharaJobTypeCLITest):

    def test_plugin_cli(self):
        self.openstack_plugin_list()
        self.openstack_plugin_show()
        self.openstack_plugin_configs_get()
        if TEMPEST_CONF.data_processing.plugin_update_support:
            self.openstack_plugin_update()

    def test_node_group_cli(self):
        master_ngt = self.openstack_node_group_template_create('master', '4')
        worker_ngt = self.openstack_node_group_template_create('worker', '3')
        self.addCleanup(self.delete_resource, NODE_GROUP_TEMPLATE, master_ngt)
        self.addCleanup(self.delete_resource, NODE_GROUP_TEMPLATE, worker_ngt)
        self.filter_node_group_list_with_plugin()
        self.openstack_node_group_template_list()
        new_master_ngt = self.openstack_node_group_template_update(
            master_ngt, update_field='name')
        self.addCleanup(self.delete_resource, NODE_GROUP_TEMPLATE,
                        new_master_ngt)

        self.openstack_node_group_template_show(new_master_ngt)
        self.openstack_node_group_template_delete(new_master_ngt)
        self.negative_try_to_delete_protected_node_group(worker_ngt)
        self.openstack_node_group_template_delete(worker_ngt)
        self.wait_for_resource_deletion(new_master_ngt, NODE_GROUP_TEMPLATE)
        self.wait_for_resource_deletion(worker_ngt, NODE_GROUP_TEMPLATE)
        self.negative_delete_removed_node_group(worker_ngt)

    def test_cluster_template_cli(self):
        cluster_template_cmd = 'cluster template'
        ng_master = (
            self.openstack_node_group_template_create('tmp-master', '4'))
        ng_worker = (
            self.openstack_node_group_template_create('tmp-worker', '3'))
        self.addCleanup(self.delete_resource, NODE_GROUP_TEMPLATE,
                        ng_master)
        self.addCleanup(self.delete_resource, NODE_GROUP_TEMPLATE,
                        ng_worker)

        cluster_template_name = (
            self.openstack_cluster_template_create(ng_master, ng_worker))
        self.addCleanup(self.delete_resource, cluster_template_cmd,
                        cluster_template_name)

        self.openstack_cluster_template_list()
        self.openstack_cluster_template_show(cluster_template_name)
        new_cluster_template_name = self.openstack_cluster_template_update(
            cluster_template_name)
        self.addCleanup(self.delete_resource, cluster_template_cmd,
                        new_cluster_template_name)

        self.openstack_cluster_template_delete(new_cluster_template_name)
        self.wait_for_resource_deletion(new_cluster_template_name,
                                        cluster_template_cmd)
        self.openstack_node_group_template_delete(ng_master)
        self.openstack_node_group_template_delete(ng_worker)
        self.wait_for_resource_deletion(ng_master, NODE_GROUP_TEMPLATE)
        self.wait_for_resource_deletion(ng_worker, NODE_GROUP_TEMPLATE)

    @decorators.skip_because(bug="1629295")
    def test_cluster_cli(self):
        image_name = self.openstack_image_register(
            TEMPEST_CONF.data_processing.test_image_name,
            TEMPEST_CONF.data_processing.test_ssh_user)
        self.openstack_image_tags_set(image_name)
        self.openstack_image_tags_remove(image_name)
        self.openstack_image_tags_add(image_name)
        self.openstack_image_show(image_name)
        self.openstack_image_list()
        flavors_client = self.client_manager_admin.flavors_client
        flavor_ref = flavors_client.create_flavor(name='sahara-flavor',
                                                  ram=512, vcpus=1, disk=4,
                                                  id=20)['flavor']
        self.addCleanup(flavors_client.delete_flavor, flavor_ref['id'])
        self.addCleanup(self.openstack_image_unregister, image_name)

        ng_master = self.openstack_node_group_template_create('cli-cluster'
                                                              '-master',
                                                              'sahara-flavor')
        ng_worker = self.openstack_node_group_template_create('cli-cluster'
                                                              '-worker',
                                                              'sahara-flavor')
        self.addCleanup(self.delete_resource, 'node group template',
                        ng_master)
        self.addCleanup(self.delete_resource, 'node group template',
                        ng_worker)

        cluster_template_name = (
            self.openstack_cluster_template_create(ng_master, ng_worker))
        self.addCleanup(self.delete_resource, 'cluster template',
                        cluster_template_name)

        cluster_name = (
            self.openstack_cluster_create(cluster_template_name, image_name))
        self.addCleanup(self.delete_resource, 'cluster',
                        cluster_name)

        self._run_job_on_cluster(cluster_name)
        self.openstack_cluster_list()
        self.openstack_cluster_show(cluster_name)
        self.openstack_cluster_update(cluster_name)
        self.openstack_cluster_verification_show(cluster_name)
        self.openstack_cluster_verification_start(cluster_name)
        self.openstack_cluster_scale(cluster_name, ng_worker)
        self.openstack_cluster_delete(cluster_name)
        self.wait_for_resource_deletion(cluster_name, 'cluster')
        self.openstack_cluster_template_delete(cluster_template_name)
        self.wait_for_resource_deletion(cluster_template_name, 'cluster '
                                                               'template')
        self.openstack_node_group_template_delete(ng_master)
        self.openstack_node_group_template_delete(ng_worker)
        self.wait_for_resource_deletion(ng_master, 'node group template')
        self.wait_for_resource_deletion(ng_worker, 'node group template')
        self.openstack_image_unregister(image_name)
        self.negative_unregister_not_existing_image(image_name)

    @testtools.skipIf(TEMPEST_CONF.data_processing.api_version_saharaclient !=
                      '1.1', "Full job binaries testing requires API v1.1")
    def test_job_binary_cli(self):
        job_binary_name, original_file = self.openstack_job_binary_create()
        self.addCleanup(self.delete_resource, 'job binary', job_binary_name)

        self.openstack_job_binary_list()
        self.openstack_job_binary_show(job_binary_name)
        self.openstack_job_binary_update(job_binary_name, flag='description')
        self.openstack_job_binary_download(job_binary_name, original_file)
        self.filter_job_binaries_in_list()
        self.negative_try_to_update_protected_jb(job_binary_name)
        self.openstack_job_binary_update(job_binary_name, flag='unprotected')
        self.openstack_job_binary_delete(job_binary_name)
        self.negative_delete_removed_job_binary(job_binary_name)

    def test_job_template_cli(self):
        job_binary_name, _ = self.openstack_job_binary_create(
            job_internal=False)
        self.addCleanup(self.delete_resource, 'job binary', job_binary_name)

        job_template_name = self.openstack_job_template_create(job_binary_name)
        self.addCleanup(self.delete_resource, 'job template',
                        job_template_name)

        self.openstack_job_template_list()
        self.openstack_job_template_show(job_template_name)
        self.openstack_job_template_update(job_template_name)
        self.openstack_job_template_delete(job_template_name)
        self.openstack_job_binary_delete(job_binary_name)

    def test_data_source_cli(self):
        data_source_name = self.openstack_data_source_create()
        self.addCleanup(self.delete_resource, 'data source', data_source_name)

        self.openstack_data_source_list()
        self.openstack_data_source_show(data_source_name)
        self.openstack_data_source_update(data_source_name)
        self.openstack_data_source_delete(data_source_name)

    def test_job_type_cli(self):
        self.openstack_job_type_list()
        self.openstack_job_type_configs_get(flag='file')
        self.filter_job_type_in_list()

    def _run_job_on_cluster(self, cluster_name):
        job_template_name = self.openstack_job_template_name()
        self.addCleanup(self.delete_resource, 'job template',
                        job_template_name)

        input_file = self.openstack_data_source_create()
        output_file = self.openstack_data_source_create()
        self.addCleanup(self.delete_resource, 'data source', input_file)
        self.addCleanup(self.delete_resource, 'data source', output_file)
        job_id = self.openstack_job_execute(cluster_name, job_template_name,
                                            input_file, output_file)
        self.addCleanup(self.delete_resource, 'job', job_id)

        self.openstack_job_list()
        self.openstack_job_show(job_id)
        self.openstack_job_update(job_id)
        self.openstack_job_delete(job_id)
        self.openstack_data_source_delete(input_file)
        self.openstack_data_source_delete(output_file)
        self.openstack_job_template_delete(job_template_name)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sahara_tempest_plugin.tests.cli import base
from tempest.lib.common.utils import data_utils


class SaharaClusterTemplateCLITest(base.ClientTestBase):

    def openstack_cluster_template_list(self):
        self.assertTableStruct(self.listing_result('cluster template list'), [
            'Name',
            'Id',
            'Plugin name',
            'Plugin version'
        ])

    def openstack_cluster_template_create(self, ng_master, ng_worker):
        cluster_template_name = data_utils.rand_name('cl-tmp')
        flag = ("%(ct_name)s %(ngm)s %(ngw)s "
                % {'ngw': ''.join([ng_worker, ':3']),
                   'ngm': ''.join([' --node-groups ', ng_master, ':1 ']),
                   'ct_name': ''.join(['--name ', cluster_template_name])})
        self.assertTableStruct(
            self.listing_result(''.join(['cluster template create ', flag])),
            [
                'Field',
                'Value'
            ])
        return cluster_template_name

    def openstack_cluster_template_show(self, cluster_template_name):
        self.find_in_listing(
            self.listing_result(
                ''.join(['cluster template show ', cluster_template_name])),
            cluster_template_name)

    def openstack_cluster_template_update(self, cluster_template_name):
        cmd = 'cluster template'
        new_template_name = self.update_resource_value(cmd,
                                                       cluster_template_name,
                                                       '--name')
        return new_template_name

    def openstack_cluster_template_delete(self, cluster_template_name):
        self.check_if_delete('cluster template', cluster_template_name)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

import mock
import testtools

from sahara_tests.scenario import utils


class UtilsGenerateConfigTest(testtools.TestCase):

    def setUp(self):
        super(UtilsGenerateConfigTest, self).setUp()
        self._cluster_config = {
            "clusters": [{
                "edp_jobs_flow": [
                    "pig_all",
                    {
                        "name": "pig_feat2",
                        "features": ["feat2"]
                    },
                    {
                        "name": "pig_featmulti",
                        "features": ["feat1", "feat2"]
                    },
                ]
            }],
            "edp_jobs_flow": {
                "pig_all": [
                    {
                        "type": "Pig",
                        "input_datasource": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/"
                                      "top-todoers/data/input"
                        },
                        "output_datasource": {
                            "type": "hdfs",
                            "destination": "/user/hadoop/edp-output"
                        },
                        "main_lib": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/"
                                      "top-todoers/example.pig"
                        }
                    }
                ],
                "pig_feat2": [
                    {
                        "type": "Pig",
                        "input_datasource": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/"
                                      "top-todoers/data/input"
                        },
                        "output_datasource": {
                            "type": "hdfs",
                            "destination": "/user/hadoop/edp-output"
                        },
                        "main_lib": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/"
                                      "top-todoers/example.pig"
                        }
                    }
                ],
                "pig_featmulti": [
                    {
                        "type": "Pig",
                        "input_datasource": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/"
                                      "top-todoers/data/input"
                        },
                        "output_datasource": {
                            "type": "hdfs",
                            "destination": "/user/hadoop/edp-output"
                        },
                        "main_lib": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/"
                                      "top-todoers/example.pig"
                        }
                    }
                ]
            }
        }

    @mock.patch('sahara_tests.scenario.utils.read_scenario_config')
    def test_generate_config_feature(self, m_readscenarioconfig):
        """Check the generate_config method when features are specified."""
        m_readscenarioconfig.return_value = self._cluster_config
        # "template_variables" can be empty because read_scenario_config,
        # which is its only users, is a mock variable.
        # "files" is used to loop over the read keys, so at one fake
        # file name is needed.
        result = utils.generate_config(['dummyfile.yaml'], None,
                                       {'credentials': {}}, False,
                                       features_list=['feat1'])
        self.assertIn('clusters', result)
        self.assertEqual(set(result['clusters'][0]['edp_jobs_flow']),
                         set(["pig_all", "pig_featmulti"]))

    @mock.patch('sahara_tests.scenario.utils.read_scenario_config')
    def test_generate_config_nofeatures(self, m_readscenarioconfig):
        """Check the generate_config method when features are specified."""
        m_readscenarioconfig.return_value = self._cluster_config
        # "template_variables" can be empty because read_scenario_config,
        # which is its only users, is a mock variable.
        # "files" is used to loop over the read keys, so at one fake
        # file name is needed.
        result = utils.generate_config(['dummyfile.yaml'], None,
                                       {'credentials': {}}, False,
                                       features_list=[])
        self.assertIn('clusters', result)
        self.assertEqual(set(result['clusters'][0]['edp_jobs_flow']),
                         set(["pig_all"]))

    @mock.patch('sahara_tests.scenario.utils.read_scenario_config')
    def test_generate_config_singlejob_str(self, m_readscenarioconfig):
        """Check the generate_config method when the cluster runs only
        a single job defined as string and not as list."""
        cluster_config_singlejob_str = self._cluster_config
        cluster_config_singlejob_str['clusters'][0]["edp_jobs_flow"] = \
            'pig_all'
        m_readscenarioconfig.return_value = cluster_config_singlejob_str
        # "template_variables" can be empty because read_scenario_config,
        # which is its only users, is a mock variable.
        # "files" is used to loop over the read keys, so at one fake
        # file name is needed.
        result = utils.generate_config(['dummyfile.yaml'], None,
                                       {'credentials': {}}, False,
                                       features_list=[])
        self.assertIn('clusters', result)
        self.assertEqual(set(result['clusters'][0]['edp_jobs_flow']),
                         set(["pig_all"]))

    @mock.patch('sahara_tests.scenario.utils.read_scenario_config')
    def test_generate_config_unknownjob(self, m_readscenarioconfig):
        """Check the generate_config method when an unknown job is specified,
        thus leading to an exception."""
        cluster_config_unknownjob = self._cluster_config
        cluster_config_unknownjob['clusters'][0]["edp_jobs_flow"].append(
            'unknown_job')
        m_readscenarioconfig.return_value = cluster_config_unknownjob
        # "template_variables" can be empty because read_scenario_config,
        # which is its only users, is a mock variable.
        # "files" is used to loop over the read keys, so at one fake
        # file name is needed.
        self.assertRaises(ValueError, utils.generate_config,
                          ['dummyfile.yaml'], None, {'credentials': {}},
                          False, features_list=[])


class UtilsTemplatesTest(testtools.TestCase):

    def test_get_default_templates_noversion_nofeatures(self):
        """Check the list of automatically discovered templates
        when the plugin does not require versions and
        there are no features specified."""
        found_templates = utils.get_default_templates('fake', None, None,
                                                      ['conffile.yaml'])
        expected_templates = (
            utils.DEFAULT_TEMPLATE_VARS +
            [os.path.join(utils.TEST_TEMPLATE_DIR, 'fake.yaml.mako'),
             'conffile.yaml']
        )
        self.assertListEqual(found_templates, expected_templates)

    def test_get_default_templates_missingversion(self):
        """Check the list of automatically discovered templates
        when the plugin requires a version but it was not specified."""
        self.assertRaises(ValueError, utils.get_default_templates,
                          'vanilla', None, None, ['conffile.yaml'])

    @mock.patch('os.path.exists', side_effect=[True, False, True, True])
    def test_get_default_templates_version_release_features(self, m_exist):
        """Check the list of automatically discovered templates
        when the plugin requires a version, a release is specified and
        there are features specified."""
        found_templates = utils.get_default_templates('vanilla', '2.7.1',
                                                      'rocky',
                                                      ['conffile.yaml'],
                                                      ['feat1', 'feat2'])
        expected_templates = (
            utils.DEFAULT_TEMPLATE_VARS +
            [os.path.join(utils.TEST_TEMPLATE_DIR, 'rocky',
                          'vanilla-2.7.1.yaml.mako'),
             os.path.join(utils.TEST_TEMPLATE_DIR,
                          'credentials_feat1.yaml.mako'),
             os.path.join(utils.TEST_TEMPLATE_DIR,
                          'credentials_feat2.yaml.mako'),
             os.path.join(utils.TEST_TEMPLATE_DIR,
                          'edp_feat2.yaml.mako'),
             'conffile.yaml']
        )
        self.assertListEqual(found_templates, expected_templates)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.clients import base


CONF = config.CONF


class JobBinaryInternalsTest(base.BaseDataProcessingTest):
    @classmethod
    def skip_checks(cls):
        super(JobBinaryInternalsTest, cls).skip_checks()
        if CONF.data_processing.api_version_saharaclient != '1.1':
            raise cls.skipException('Job binaries stored on the internal db '
                                    'are available only with API v1.1')

    def _check_job_binary_internal_create(self):
        name = data_utils.rand_name('sahara-internal-job-binary')
        self.job_binary_data = b'Some data'
        # create job binary internal
        resp_body = self.create_job_binary_internal(name, self.job_binary_data)
        # check that job_binary_internal created successfully
        self.assertEqual(name, resp_body.name)
        return resp_body.id, resp_body.name

    def _check_job_binary_internal_list(self, binary_id, binary_name):
        # check for job binary internal in list
        binary_list = self.client.job_binary_internals.list()
        binaries_info = [(binary.id, binary.name) for binary in binary_list]
        self.assertIn((binary_id, binary_name), binaries_info)

    def _check_job_binary_internal_get(self, binary_id, binary_name):
        # check job binary internal fetch by id
        binary = self.client.job_binary_internals.get(binary_id)
        self.assertEqual(binary_name, binary.name)

    def _check_job_binary_internal_update(self, binary_id):
        values = {
            'name': data_utils.rand_name('sahara-internal-job-binary'),
            'is_public': True
        }
        binary = self.client.job_binary_internals.update(binary_id, **values)
        self.assertDictContainsSubset(values, binary.job_binary_internal)

    def _check_job_binary_internal_delete(self, binary_id):
        # delete job binary internal by id
        self.client.job_binary_internals.delete(binary_id)
        # check that job binary internal really deleted
        binary_list = self.client.job_binary_internals.list()
        self.assertNotIn(binary_id, [binary.id for binary in binary_list])

    def test_job_binary_internal(self):
        binary_id, binary_name = self._check_job_binary_internal_create()
        self._check_job_binary_internal_list(binary_id, binary_name)
        self._check_job_binary_internal_get(binary_id, binary_name)
        self._check_job_binary_internal_update(binary_id)
        self._check_job_binary_internal_delete(binary_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.cli import base


class SaharaJobTemplateCLITest(base.ClientTestBase):

    def openstack_job_template_list(self):
        self.assertTableStruct(self.listing_result('job template list'), [
            'Name',
            'Id',
            'Type'
        ])

    def openstack_job_template_create(self, job_binary_name):
        job_template_name = data_utils.rand_name('job-template')
        flag = ("%(name)s %(type)s %(main)s "
                % {'name': ' --name %s' % job_template_name,
                   'type': ' --type Shell',
                   'main': ' --main %s' % job_binary_name})
        self.assertTableStruct(
            self.listing_result('job template create %s' % flag),
            [
                'Field',
                'Value'
            ])
        return job_template_name

    def openstack_job_template_show(self, job_template_name):
        self.find_in_listing(
            self.listing_result('job template show %s' % job_template_name),
            job_template_name)

    def openstack_job_template_update(self, job_template_name):
        self.assertTableStruct(
            self.listing_result('job template update --description '
                                'cli-tests %s' % job_template_name), [
                'Field',
                'Value'
            ])

    def openstack_job_template_delete(self, job_template_name):
        self.check_if_delete('job template', job_template_name)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import argparse
import collections
import os
import shutil
import subprocess
import sys
import tempfile

from mako import template as mako_template
from oslo_utils import fileutils
from oslo_utils import uuidutils
import os_client_config
import pkg_resources as pkg
import six
import yaml

from sahara_tests.scenario import validation
from sahara_tests import version


SCENARIO_RESOURCES_DIR = pkg.resource_filename(version.version_info.package,
                                               'scenario')

TEST_TEMPLATE_DIR = os.path.join(SCENARIO_RESOURCES_DIR, 'defaults/')
DEFAULT_TEMPLATE_VARS = [os.path.join(TEST_TEMPLATE_DIR,
                                      'credentials.yaml.mako'),
                         os.path.join(TEST_TEMPLATE_DIR,
                                      'edp.yaml.mako')]
TEST_TEMPLATE_PATH = os.path.join(SCENARIO_RESOURCES_DIR,
                                  'testcase.py.mako')
DEFAULT_STESTR_CONF = os.path.join(SCENARIO_RESOURCES_DIR, 'stestr.conf')


def rand_name(name=''):
    rand_data = uuidutils.generate_uuid()[:8]
    if name:
        return '%s-%s' % (name, rand_data)
    else:
        return rand_data


def run_tests(concurrency, test_dir_path):
    command = ['stestr', 'run']
    if concurrency:
        command.extend(['--concurrency=%d' % concurrency])
    new_env = os.environ.copy()
    tester_runner = subprocess.Popen(command, env=new_env, cwd=test_dir_path)
    tester_runner.communicate()
    return tester_runner.returncode


def create_testcase_file(testcases, credentials, network, report,
                         use_api_v2=False):
    # current directory, where to write reports, key files, etc, if required
    results_dir = os.getcwd()
    default_templ_dir = os.path.abspath(TEST_TEMPLATE_DIR)

    # create testcase file
    test_template = mako_template.Template(filename=TEST_TEMPLATE_PATH)
    testcase_data = test_template.render(testcases=testcases,
                                         credentials=credentials,
                                         network=network, report=report,
                                         results_dir=results_dir,
                                         default_templ_dir=default_templ_dir,
                                         use_api_v2=use_api_v2)

    test_dir_path = tempfile.mkdtemp()
    print("The generated test file located at: %s" % test_dir_path)
    fileutils.write_to_tempfile(testcase_data.encode("ASCII"), prefix='test_',
                                suffix='.py', path=test_dir_path)
    # Copy both files as long as the old runner is supported
    shutil.copyfile(DEFAULT_STESTR_CONF, os.path.join(test_dir_path,
                                                      '.stestr.conf'))

    return test_dir_path


def get_templates_variables(files, variable_file, verbose_run, scenario_args,
                            auth_values):
    template_variables = {}
    if any(is_template_file(config_file) for config_file in files):
        template_variables = read_template_variables(variable_file,
                                                     verbose_run,
                                                     scenario_args)
    template_variables.update(read_template_variables(
        verbose=verbose_run, scenario_args=scenario_args,
        auth_values=auth_values))
    return template_variables


def generate_config(files, template_variables, auth_values, verbose_run,
                    features_list=None):
    config = {'credentials': {},
              'network': {},
              'clusters': [],
              'edp_jobs_flow': {}}

    for scenario_argument in files:
        test_scenario = read_scenario_config(scenario_argument,
                                             template_variables, verbose_run)
        config = _merge_dicts_sections(test_scenario, config, 'credentials')
        config = _merge_dicts_sections(test_scenario, config, 'network')

        if test_scenario.get('clusters') is not None:
            config['clusters'] += test_scenario['clusters']

        if test_scenario.get('edp_jobs_flow') is not None:
            for key in test_scenario['edp_jobs_flow']:
                if key not in config['edp_jobs_flow']:
                    config['edp_jobs_flow'][key] = (
                        test_scenario['edp_jobs_flow'][key])
                else:
                    raise ValueError('Job flow exist')
    config['credentials'].update(auth_values['credentials'])

    # filter out the jobs depending on the features, if any
    unknown_jobs = []
    if features_list is None:
        features_list = []
    for cluster in config['clusters']:
        if cluster.get('edp_jobs_flow'):
            filtered_jobs = []
            # The jobs associated to a cluster may be a single value (string)
            # or a list of values; handle both cases.
            cluster_jobs_list = cluster['edp_jobs_flow']
            if isinstance(cluster_jobs_list, six.string_types):
                cluster_jobs_list = [cluster_jobs_list]
            for job_item in cluster_jobs_list:
                if isinstance(job_item, dict):
                    job = job_item['name']
                else:
                    job = job_item

                # get the list of features, if defined
                job_features = set()
                if isinstance(job_item, dict):
                    job_features = set(job_item.get('features', []))
                # If a job has no features associated, it is always used.
                # Otherwise, it should be used only if any of its features
                # matches any of the features requested,
                if (not job_features or
                    (features_list is not None and
                     job_features.intersection(features_list))):
                    # the job is relevant for the configuration,
                    # so it must be defined; if it is not, it will break,
                    # so take note of the name
                    if job not in config['edp_jobs_flow']:
                        unknown_jobs.append(job)
                        continue
                    # job defined, it can be used
                    filtered_jobs.append(job)

            cluster['edp_jobs_flow'] = filtered_jobs
    if unknown_jobs:
        # Some jobs which are listed in some clusters
        # are not defined, stop here
        raise ValueError('Unknown jobs: %s' % (unknown_jobs))

    if verbose_run:
        six.print_("Generated configuration:\n%s" % (
            yaml.safe_dump(config,
                           allow_unicode=True,
                           default_flow_style=False)),
                   flush=True)
    return config


def get_default_templates(plugin, version, release, scenario_arguments,
                          features=None):
    all_templates = []

    templates_location = TEST_TEMPLATE_DIR
    if release is not None:
        templates_location = os.path.join(TEST_TEMPLATE_DIR, release)

    if plugin:
        if plugin in ['transient', 'fake']:
            template = "%s.yaml.mako" % plugin
        elif plugin and version:
            template = "%s-%s.yaml.mako" % (plugin, version)
        else:
            raise ValueError("Please, specify version for plugin via '-v'")

        # find the templates for each features, if they exist
        feature_templates_vars = []
        if features:
            default_templates_base = []
            for default_template in DEFAULT_TEMPLATE_VARS:
                if default_template.endswith('.yaml.mako'):
                    default_templates_base.append(
                        default_template[:-len('.yaml.mako')])
            # for each default template, look for a corresponding
            # <file>_<feature>.yaml.mako
            for feature in features:
                for default_template_base in default_templates_base:
                    template_feature = '%s_%s.yaml.mako' % (
                        default_template_base, feature)
                    if os.path.exists(template_feature):
                        feature_templates_vars.append(template_feature)

        # return a combination of: default templates, version-specific
        # templates, feature-based templates
        all_templates = DEFAULT_TEMPLATE_VARS + [os.path.join(
            templates_location, template)] + feature_templates_vars

    # it makes sense that all the other templates passed as arguments
    # are always added at the end
    all_templates += scenario_arguments
    return all_templates


def get_auth_values(cloud_config, args):
    try:
        cloud = cloud_config.get_one_cloud(argparse=args)
        cloud_credentials = cloud.get_auth_args()
        api_version = cloud.config.get('identity_api_version')
    except os_client_config.exceptions.OpenStackConfigException:
        # cloud not found
        api_version = '2.0'
        cloud_credentials = {}
    auth_values = {
        'os_username': cloud_credentials.get('username', 'admin'),
        'os_password': cloud_credentials.get('password', 'nova'),
        'os_auth_url': cloud_credentials.get('auth_url',
                                             'http://localhost:5000/v2.0'),
        'os_tenant': cloud_credentials.get('project_name', 'admin')
    }
    auth_url = auth_values['os_auth_url']
    if not any(v in auth_url for v in ('v2.0', 'v3')):
        version = 'v3' if api_version in ('3', '3.0') else 'v2.0'
        auth_values['os_auth_url'] = "%s/%s" % (auth_url, version)
    return auth_values


def _merge_dicts_sections(dict_with_section, dict_for_merge, section):
    if dict_with_section.get(section) is not None:
        for key in dict_with_section[section]:
            if dict_for_merge[section].get(key) is not None:
                if dict_for_merge[section][key] != (
                        dict_with_section[section][key]):
                    raise ValueError('Sections %s is different' % section)
            else:
                dict_for_merge[section][key] = dict_with_section[section][key]
    return dict_for_merge


def is_template_file(config_file):
    return config_file.endswith(('.yaml.mako', '.yml.mako'))


def read_template_variables(variable_file=None, verbose=False,
                            scenario_args=None, auth_values=None):
    variables = {}
    try:
        cp = six.moves.configparser.ConfigParser()
        # key-sensitive keys
        if variable_file:
            cp.optionxform = lambda option: option
            cp.readfp(open(variable_file))
        variables = cp.defaults()
        if scenario_args:
            variables.update(scenario_args)
        if auth_values:
            variables.update(auth_values)
    except IOError as ioe:
        print("WARNING: the input contains at least one template, but "
              "the variable configuration file '%s' is not valid: %s" %
              (variable_file, ioe))
    except six.moves.configparser.Error as cpe:
        print("WARNING: the input contains at least one template, but "
              "the variable configuration file '%s' can not be parsed: "
              "%s" % (variable_file, cpe))
    finally:
        if verbose:
            six.print_("Template variables:\n%s" % (variables), flush=True)
    # continue anyway, as the templates could require no variables
    return variables


def read_scenario_config(scenario_config, template_vars=None,
                         verbose=False):
    """Parse the YAML or the YAML template file.

    If the file is a YAML template file, expand it first.
    """

    yaml_file = ''
    if is_template_file(scenario_config):
        scenario_template = mako_template.Template(filename=scenario_config,
                                                   strict_undefined=True)
        template = scenario_template.render_unicode(**template_vars)
        yaml_file = yaml.safe_load(template)
    else:
        with open(scenario_config, 'r') as yaml_file:
            yaml_file = yaml.safe_load(yaml_file)
    if verbose:
        six.print_("YAML from %s:\n%s" % (scenario_config,
                                          yaml.safe_dump(
                                              yaml_file,
                                              allow_unicode=True,
                                              default_flow_style=False)),
                   flush=True)
    return yaml_file



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.clients import base


class ClusterTemplateTest(base.BaseDataProcessingTest):
    def _check_create_cluster_template(self):
        ng_template_name = data_utils.rand_name('sahara-ng-template')
        ng_template = self.create_node_group_template(ng_template_name,
                                                      **self.worker_template)

        full_cluster_template = self.cluster_template.copy()

        # The 'node_groups' field in the response body
        # has some extra info that post body does not have.
        del self.cluster_template['node_groups']

        template_name = data_utils.rand_name('sahara-cluster-template')

        # create cluster template
        resp_body = self.create_cluster_template(template_name,
                                                 **full_cluster_template)

        # check that template created successfully
        self.assertEqual(template_name, resp_body.name)
        self.assertDictContainsSubset(self.cluster_template,
                                      resp_body.__dict__)

        return resp_body.id, template_name

    def _check_cluster_template_list(self, template_id, template_name):
        # check for cluster template in list
        template_list = self.client.cluster_templates.list()
        templates_info = [(template.id, template.name)
                          for template in template_list]
        self.assertIn((template_id, template_name), templates_info)

    def _check_cluster_template_get(self, template_id, template_name):
        # check cluster template fetch by id
        template = self.client.cluster_templates.get(
            template_id)
        self.assertEqual(template_name, template.name)
        self.assertDictContainsSubset(self.cluster_template, template.__dict__)

    def _check_cluster_template_update(self, template_id):
        values = {
            'name': data_utils.rand_name('updated-sahara-ct'),
            'description': 'description',
        }

        # check updating of cluster template
        template = self.client.cluster_templates.update(
            template_id, **values)
        self.assertDictContainsSubset(values, template.__dict__)

    def _check_cluster_template_delete(self, template_id):
        # delete cluster template by id
        self.client.cluster_templates.delete(
            template_id)

        # check that cluster template really deleted
        templates = self.client.cluster_templates.list()
        self.assertNotIn(template_id, [template.id for template in templates])

    def test_cluster_templates(self):
        template_id, template_name = self._check_create_cluster_template()
        self._check_cluster_template_list(template_id, template_name)
        self._check_cluster_template_get(template_id, template_name)
        self._check_cluster_template_update(template_id)
        self._check_cluster_template_delete(template_id)
